from typing import Literal, Optional

from fireworks._literals import AcceleratorTypeLiteral, ReinforcementAcceleratorTypeLiteral
from fireworks._util import generate_model_resource_name
from fireworks.control_plane.generated.protos_grpcio.gateway.status_pb2 import JobState
from fireworks.dataset.dataset import Dataset
from fireworks.gateway import Gateway
from fireworks.llm.llm import LLM
from fireworks.control_plane.generated.protos_grpcio.gateway.deployment_pb2 import (
    AcceleratorType as SyncAcceleratorType,
)
from fireworks.control_plane.generated.protos_grpcio.gateway.rlor_trainer_job_pb2 import (
    RlorTrainerJob as RlorTrainerJobProto,
    CreateRlorTrainerJobRequest as CreateRlorTrainerJobRequestProto,
)
from fireworks.control_plane.generated.protos_grpcio.gateway.training_pb2 import (
    BaseTrainingConfig as BaseTrainingConfigProto,
)


class LLMReinforcementStep:
    """
    Class that provides reinforcement learning step functionality.
    This uses composition instead of inheritance for cleaner separation.
    """

    def __init__(self, llm: LLM):
        """
        Initialize with an LLM instance.

        Args:
            llm: The LLM instance to work with
        """
        self.llm = llm

    def reinforcement_step(
        self,
        dataset: Dataset,
        output_model: str,
        lora_rank: int = 16,
        learning_rate: float = 0.0001,
        max_context_length: int = 8192,
        epochs: int = 1,
        batch_size: int = 32768,
        accelerator_count: int = 1,
        accelerator_type: ReinforcementAcceleratorTypeLiteral = "NVIDIA_H200_141GB",
    ) -> "ReinforcementStep":
        """
        Perform a reinforcement learning step with the given dataset.

        Args:
            dataset: The dataset to use for the reinforcement learning step

        Returns:
            Dict containing the step results and any additional information
        """
        # check if the model already exists
        account_id = self.llm._gateway.account_id()
        model = self.llm._gateway.get_model_sync(generate_model_resource_name(account_id, output_model))
        if model is not None:
            raise ValueError(f"Model {output_model} already exists")

        training_config = BaseTrainingConfigProto()
        training_config.output_model = f"accounts/{self.llm._gateway.account_id()}/models/{output_model}"
        if self.llm.is_peft_addon():
            training_config.warm_start_from = self.llm.model
        else:
            training_config.base_model = self.llm.model
        training_config.lora_rank = lora_rank
        training_config.learning_rate = learning_rate
        training_config.max_context_length = max_context_length
        training_config.epochs = epochs
        training_config.batch_size = batch_size
        training_config.accelerator_type = getattr(SyncAcceleratorType, accelerator_type)
        training_config.accelerator_count = accelerator_count
        rlor_trainer_job = RlorTrainerJobProto(
            training_config=training_config,
            dataset=dataset.name,
        )
        create_rlor_trainer_job_request = CreateRlorTrainerJobRequestProto(
            rlor_trainer_job=rlor_trainer_job,
        )
        job = self.llm._gateway.create_rlor_trainer_job_sync(create_rlor_trainer_job_request)
        return ReinforcementStep(job, self.llm._gateway)


class ReinforcementStep:
    """
    Class that represents a RLOR trainer job.
    This is a wrapper around the RlorTrainerJobProto class.
    """

    def __init__(self, job: RlorTrainerJobProto, gateway: Gateway):
        self.job = job
        self.gateway = gateway

    def get(self) -> Optional["ReinforcementStep"]:
        job = self.gateway.get_rlor_trainer_job_sync(self.job.name)
        if job is None:
            return None
        return ReinforcementStep(job, self.gateway)

    @property
    def state(self) -> str:
        return JobState.Name(self.job.state)

    @property
    def output_model(self):
        return self.job.training_config.output_model

    @property
    def is_completed(self):
        return self.job.state == JobState.JOB_STATE_COMPLETED

    def raise_if_bad_state(self):
        """
        Raises a RuntimeError if the job is in a failed, cancelled, or otherwise bad state.
        """
        bad_states = {
            JobState.JOB_STATE_FAILED,
            JobState.JOB_STATE_FAILED_CLEANING_UP,
            JobState.JOB_STATE_CANCELLED,
            JobState.JOB_STATE_EXPIRED,
            JobState.JOB_STATE_EXPIRED_CLEANING_UP,
            JobState.JOB_STATE_DELETING_CLEANING_UP,
        }
        if self.state in bad_states:
            raise RuntimeError(f"RLORTrainerJob is in a bad state: {self.state}. Message: {self.job.status.message}")



import asyncio
import atexit
import json
import aiohttp
import httpx
from typing import Callable, Dict, Any, Optional, Generator, Union, AsyncGenerator
import httpx_sse
from .._util import is_running_in_async_context
from .error import (
    AuthenticationError,
    APITimeoutError,
    BadGatewayError,
    InternalServerError,
    InvalidRequestError,
    PermissionError,
    RateLimitError,
    ServiceUnavailableError,
)


# Helper functions for api key and base url prevent cyclic dependencies
def default_api_key() -> str:
    from fireworks.client import api_key

    if api_key is not None:
        return api_key
    else:
        raise ValueError(
            "No API key provided. You can set your API key in code using 'fireworks.client.api_key = <API-KEY>', or you can set the environment variable FIREWORKS_API_KEY=<API-KEY>)."
        )


def default_base_url() -> str:
    from fireworks.client import base_url

    return base_url


# Based on benchmark results, 1000 connections is a good balance between
# performance and not overloading the server.
DEFAULT_AIOHTTP_CONNECTOR_LIMIT = 1_000


class FireworksClient:
    """
    Fireworks client class for handling HTTP requests to the Fireworks API.

    Supports both synchronous and asynchronous operations with streaming and
    non-streaming responses. Uses httpx for primary HTTP operations and aiohttp
    as an alternative for specific async use cases since aiohttp is better in
    highly concurrent workloads, which is important when maximizing performance
    for dedicated deployments on Fireworks.
    """

    def __init__(
        self,
        request_timeout=600,
        *,
        api_key: Union[str, None] = None,
        base_url: Union[str, httpx.URL, None] = None,
        extra_headers: Optional[Dict[str, str]] = None,
        max_connections: Optional[int] = None,
        **kwargs,
    ) -> None:
        """Initialize the Fireworks client.

        Args:
            request_timeout: Request timeout in seconds. Defaults to 600.
            api_key: Fireworks API key. If not provided, will use the global api_key.
            base_url: Base URL for API requests. If not provided, will use the global base_url.
            extra_headers: Additional headers to include in all requests.
            **kwargs: Additional arguments passed to the underlying HTTP clients.
        """
        if "request_timeout" in kwargs:
            request_timeout = kwargs["request_timeout"]
        self.api_key = api_key or default_api_key()
        if not self.api_key:
            raise AuthenticationError(
                "No API key provided. You can set your API key in code using 'fireworks.client.api_key = <API-KEY>', or you can set the environment variable FIREWORKS_API_KEY=<API-KEY>)."
            )
        self.base_url = base_url or default_base_url()
        self.request_timeout = request_timeout
        self.max_connections = max_connections
        self.client_kwargs = kwargs
        self.extra_headers = extra_headers or {}
        self._client: httpx.Client = self.create_client(extra_headers=self.extra_headers)
        self._async_client: httpx.AsyncClient = self.create_async_client(extra_headers=self.extra_headers)
        self._aiohttp_session: Optional[aiohttp.ClientSession] = self.create_aiohttp_client_session(
            extra_headers=self.extra_headers
        )

    def _raise_for(self, status_code: int, error_message: Callable[[], str]):
        if status_code == 400:
            raise InvalidRequestError(error_message())
        elif status_code == 401:
            raise AuthenticationError(error_message())
        elif status_code == 403:
            raise PermissionError(error_message())
        elif status_code == 404:
            raise InvalidRequestError(error_message())
        elif status_code == 408:
            raise APITimeoutError(error_message())
        elif status_code == 429:
            raise RateLimitError(error_message())
        elif status_code == 500:
            raise InternalServerError(error_message())
        elif status_code == 502:
            raise BadGatewayError(error_message())
        elif status_code == 503:
            raise ServiceUnavailableError(error_message())

    def _raise_for_status(self, response: Union[httpx.Response, aiohttp.ClientResponse], response_text: str = ""):
        """
        Raises an error if the response status code is not 200. Handles both
        httpx and aiohttp responses.

        Args:
            response: The response object from the HTTP client.
            response_text: The text of the response.
        """

        # Function to get error message or default to response code name
        def get_error_message() -> str:
            # Try to get error message from response_text first (for aiohttp)
            if response_text:
                try:
                    # Validate it's valid JSON
                    json.loads(response_text)
                    return response_text
                except (json.JSONDecodeError, ValueError):
                    pass

            # Try to get error message from response.json() (for httpx)
            try:
                if isinstance(response, httpx.Response):
                    return json.dumps(response.json())
                else:
                    # since we can't call "await" on response.json() for aiohttp, return response_text if non-empty
                    if response_text:
                        return response_text
            except (json.JSONDecodeError, AttributeError):
                pass

            # If JSON parsing fails, return the HTTP status code name
            if isinstance(response, httpx.Response):
                status_code = response.status_code
                reason = response.reason_phrase
            else:
                status_code = response.status
                reason = response.reason

            if 400 <= status_code < 500:
                error_type = "invalid_request_error"
            elif 500 <= status_code < 600:
                error_type = "internal_server_error"
            else:
                error_type = "unknown_error"
            return json.dumps(
                {
                    "error": {
                        "object": "error",
                        "type": error_type,
                        "message": reason,
                    }
                }
            )

        status_code = response.status_code if isinstance(response, httpx.Response) else response.status
        self._raise_for(status_code, get_error_message)

        # if our implementation did not catch the error, let aiohttp or httpx handle it
        response.raise_for_status()

    async def _async_error_handling(
        self, resp: Union[httpx.Response, aiohttp.ClientResponse], response_text: str = ""
    ):
        # Handle httpx async responses
        if isinstance(resp, httpx.Response) and resp.is_error:
            await resp.aread()
        # Handle aiohttp responses
        elif isinstance(resp, aiohttp.ClientResponse) and resp.status >= 400:
            response_text = await resp.text()
        self._raise_for_status(resp, response_text)

    def _error_handling(self, resp):
        if resp.is_error:
            resp.read()
        self._raise_for_status(resp)

    def _get_request(self, url: str, extra_headers: Optional[Dict[str, str]] = None) -> Dict[str, Any]:
        resp = self._client.get(url, headers=self._build_headers(extra_headers))
        self._error_handling(resp)
        return resp.json()

    def _build_headers(self, extra_headers: Optional[Dict[str, str]] = None) -> Dict[str, str]:
        """Build headers with authorization and any extra headers.

        Args:
            extra_headers: Additional headers to merge with the authorization header and
                          instance extra_headers.

        Returns:
            Dictionary containing the Authorization header and any extra headers.
        """
        headers = {"Authorization": f"Bearer {self.api_key}"}
        # Start with instance extra_headers
        headers.update(self.extra_headers)
        # Then add any additional headers passed to this method
        if extra_headers:
            headers.update(extra_headers)
        return headers

    def post_request_streaming(
        self,
        url: str,
        data: Optional[Dict[str, Any]] = None,
        extra_headers: Optional[Dict[str, str]] = None,
    ) -> Generator[str, None, None]:
        """Make a streaming POST request.

        Args:
            url: The endpoint URL to send the request to.
            data: JSON data to send in the request body.
            extra_headers: Additional headers to include in the request.

        Yields:
            Server-sent event data as strings.
        """
        with httpx_sse.connect_sse(
            self._client,
            url=url,
            method="POST",
            json=data,
            headers=self._build_headers(extra_headers),
        ) as event_source:
            self._error_handling(event_source.response)
            for sse in event_source.iter_sse():
                yield sse.data

    def post_request_non_streaming(
        self,
        url: str,
        data: Optional[Dict[str, Any]] = None,
        extra_headers: Optional[Dict[str, str]] = None,
    ) -> Dict[str, Any]:
        """Make a non-streaming POST request.

        Args:
            url: The endpoint URL to send the request to.
            data: JSON data to send in the request body.
            extra_headers: Additional headers to include in the request.

        Returns:
            JSON response data as a dictionary.
        """
        response = self._client.post(url, json=data, headers=self._build_headers(extra_headers))
        self._error_handling(response)
        return response.json()

    async def post_request_async_streaming(
        self,
        url: str,
        data: Optional[Dict[str, Any]] = None,
        extra_headers: Optional[Dict[str, str]] = None,
    ) -> AsyncGenerator[str, None]:
        """Make an async streaming POST request.

        Args:
            url: The endpoint URL to send the request to.
            data: JSON data to send in the request body.
            extra_headers: Additional headers to include in the request.

        Yields:
            Server-sent event data as strings.
        """
        async with httpx_sse.aconnect_sse(
            self._async_client,
            url=url,
            method="POST",
            json=data,
            headers=self._build_headers(extra_headers),
        ) as event_source:
            await self._async_error_handling(event_source.response)
            async for sse in event_source.aiter_sse():
                yield sse.data

    async def post_request_async_non_streaming(
        self,
        url: str,
        data: Optional[Dict[str, Any]] = None,
        extra_headers: Optional[Dict[str, str]] = None,
    ) -> Dict[str, Any]:
        """Make an async non-streaming POST request using httpx.

        Args:
            url: The endpoint URL to send the request to.
            data: JSON data to send in the request body.
            extra_headers: Additional headers to include in the request.

        Returns:
            JSON response data as a dictionary.
        """
        # Check if we're in an async context with a running event loop
        if is_running_in_async_context() and self._aiohttp_session is None:
            self._aiohttp_session = self.create_aiohttp_client_session(extra_headers)
        if self._aiohttp_session is None:
            raise ValueError("Aiohttp client session is not initialized, likely due to a non-async context")
        async with self._aiohttp_session.post(url, json=data, headers=self._build_headers(extra_headers)) as resp:
            await self._async_error_handling(resp)
            return await resp.json()

    def __enter__(self):
        return self

    def __exit__(self, *exc):
        self.close()

    def close(self):
        self._client.close()

    async def __aenter__(self):
        return self

    async def __aexit__(self, *exc):
        await self.aclose()

    async def aclose(self):
        await self._async_client.aclose()
        if self._aiohttp_session is not None:
            await self._aiohttp_session.close()

    def create_client(self, extra_headers: Optional[Dict[str, str]] = None) -> httpx.Client:
        return httpx.Client(
            headers=self._build_headers(extra_headers),
            timeout=self.request_timeout,
            **self.client_kwargs,
        )

    def create_async_client(self, extra_headers: Optional[Dict[str, str]] = None) -> httpx.AsyncClient:
        return httpx.AsyncClient(
            headers=self._build_headers(extra_headers),
            timeout=self.request_timeout,
            **self.client_kwargs,
        )

    def create_aiohttp_client_session(
        self, extra_headers: Optional[Dict[str, str]] = None
    ) -> Optional[aiohttp.ClientSession]:
        # Check if we're in an async context with a running event loop
        if not is_running_in_async_context():
            return None

        max_connections = self.max_connections or DEFAULT_AIOHTTP_CONNECTOR_LIMIT
        connector = aiohttp.TCPConnector(limit=max_connections)
        timeout = aiohttp.ClientTimeout(total=self.request_timeout)
        return aiohttp.ClientSession(
            connector=connector,
            headers=self._build_headers(extra_headers),
            timeout=timeout,
            **self.client_kwargs,
        )



import asyncio
from copy import deepcopy
import time
from typing import Optional, Union, TYPE_CHECKING
from fireworks._const import FIREWORKS_API_BASE_URL
from fireworks._util import generate_time_str
from fireworks.gateway import Gateway
from fireworks.dataset import Dataset
from fireworks.control_plane.generated.protos.gateway import (
    JobState,
)
from fireworks.control_plane.generated.protos_grpcio.gateway.supervised_fine_tuning_job_pb2 import (
    ListSupervisedFineTuningJobsRequest as SyncListSupervisedFineTuningJobsRequest,
    CreateSupervisedFineTuningJobRequest as SyncCreateSupervisedFineTuningJobRequest,
    SupervisedFineTuningJob as SyncSupervisedFineTuningJob,
)
from fireworks.control_plane.generated.protos_grpcio.gateway.status_pb2 import (
    JobState as SyncJobState,
)
from fireworks._logger import logger

# Type checking imports to avoid circular imports
if TYPE_CHECKING:
    from fireworks.llm.llm import LLM


class SupervisedFineTuningJob:
    """
    Wrapper around proto for a supervised fine-tuning job in Fireworks. Can be
    constructed from a name, LLM, and dataset. Can be used to sync the job state
    to Fireworks and query the current state.
    """

    def __init__(
        self,
        llm: "LLM",
        proto: SyncSupervisedFineTuningJob,
        dataset_or_id: Union[Dataset, str],
        evaluation_dataset_or_id: Optional[Union[Dataset, str]] = None,
        api_key: Optional[str] = None,
    ):
        """
        Args:
            llm: The LLM object that created this job.
            proto: The proto object that represents the fine-tuning job.
            dataset_or_id: The dataset or dataset ID to use for the fine-tuning job.
            evaluation_dataset: The evaluation dataset or dataset ID to use for the fine-tuning job.
            api_key: The API key to use for the fine-tuning job.
        """
        self.llm = llm
        self.dataset_or_id = dataset_or_id
        self.evaluation_dataset_or_id = evaluation_dataset_or_id
        self._api_key = api_key
        self._gateway = Gateway(api_key=api_key)
        self._proto = proto

    @property
    def output_model(self) -> Optional[str]:
        if self._proto is None:
            return None
        if self._proto.output_model is None:
            return None
        if self._proto.output_model.startswith("accounts/"):
            return self._proto.output_model
        return f"accounts/{self._gateway.account_id()}/models/{self._proto.output_model}"

    @classmethod
    async def adelete_by_name(cls, name: str, api_key: Optional[str] = None):
        gateway = Gateway(api_key=api_key)
        await gateway.delete_supervised_fine_tuning_job(name)
        job = await gateway.get_supervised_fine_tuning_job(name)
        while job is not None:
            await asyncio.sleep(1)

    @classmethod
    def delete_by_name(cls, name: str, api_key: Optional[str] = None):
        gateway = Gateway(api_key=api_key)
        gateway.delete_supervised_fine_tuning_job_sync(name)
        job = gateway.get_supervised_fine_tuning_job_sync(name)
        while job is not None:
            time.sleep(1)

    def delete(self):
        if self._proto is None:
            raise ValueError("Call sync() before deleting")
        self.delete_by_name(self._proto.name, self._api_key)

    async def adelete(self):
        if self._proto is None:
            raise ValueError("Call sync() before deleting")
        await self.adelete_by_name(self._proto.name, self._api_key)

    def sync(self) -> "SupervisedFineTuningJob":
        """
        Creates the job if it doesn't exist, otherwise returns the existing job.
        If previous job failed, deletes it and creates a new one.
        """
        # Make sure the datasets are synced before creating the job
        if isinstance(self.dataset_or_id, Dataset):
            self.dataset_or_id.sync()
        if isinstance(self.evaluation_dataset_or_id, Dataset):
            self.evaluation_dataset_or_id.sync()

        existing_job = self.get()
        if existing_job is not None:
            if (
                existing_job._proto.state == JobState.FAILED
                or existing_job._proto.state == JobState.FAILED_CLEANING_UP
                or existing_job._proto.state == JobState.CANCELLED
                or existing_job._proto.state == JobState.DELETING
                or existing_job._proto.state == JobState.DELETING_CLEANING_UP
                or existing_job._proto.state == JobState.UNSPECIFIED
            ):
                self.delete()
            else:
                return existing_job
        request = self._create_request()
        self._gateway.create_supervised_fine_tuning_job_sync(request)
        new_job = self.get()
        if new_job is None:
            raise ValueError(f"Failed to create supervised fine-tuning job {self.display_name}")
        return new_job

    @property
    def id(self) -> str:
        if self._proto is None:
            raise ValueError("Call sync() before accessing id")
        return self._proto.name.split("/")[-1]

    @property
    def display_name(self) -> str:
        if self._proto is None:
            raise ValueError("Call sync() before accessing display_name")
        return self._proto.display_name

    @property
    def name(self) -> str:
        if self._proto is None:
            raise ValueError("Call sync() before accessing name")
        return self._proto.name

    @property
    def url(self) -> str:
        base_url = "dev.fireworks.ai" if "dev." in FIREWORKS_API_BASE_URL else "app.fireworks.ai"
        if self.id is None:
            return f"https://{base_url}/dashboard/fine-tuning"
        return f"https://{base_url}/dashboard/fine-tuning/supervised/{self.id}"

    def _create_request(self) -> SyncCreateSupervisedFineTuningJobRequest:
        
        # Gateway doesn't allow you to have the name field set, so we need to remove it
        # before sending the request. But self.id should be equal to the id in the name field
        # so everything should match up with the current state of self.name
        proto = deepcopy(self._proto)
        proto.name = ""

        request = SyncCreateSupervisedFineTuningJobRequest(
            supervised_fine_tuning_job=proto,
            supervised_fine_tuning_job_id=self.id,
        )
        return request

    @property
    def output_llm(self) -> "LLM":
        # Import here to avoid circular import
        from fireworks.llm.llm import LLM

        if self.output_model is None:
            raise ValueError(f'Fine-tuning job "{self.display_name}" did not create an output model')
        if self.llm.addons_enabled:
            return LLM(model=self.output_model, deployment_type="on-demand-lora", base_id=self.llm.deployment_id)
        return LLM(model=self.output_model, deployment_type=self.llm.deployment_type, base_id=self.llm.deployment_id)

    def wait_for_completion(self) -> "SupervisedFineTuningJob":
        """
        Synchronously poll for job completion.
        """
        while self._proto.state != JobState.COMPLETED:
            if self._proto.state == JobState.FAILED:
                raise ValueError(f'Fine-tuning job "{self.display_name}" failed')
            if self._proto.create_time is not None:
                curr_time = time.time()
                create_time = self._proto.create_time.seconds + self._proto.create_time.nanos / 1e9
                delta_seconds = int(curr_time - create_time)
                minutes = delta_seconds // 60
                seconds = delta_seconds % 60
                logger.debug(
                    f'Fine-tuning job "{self.display_name}" is in state {SyncJobState.Name(self._proto.state)}. Job was created {generate_time_str(minutes, seconds)} ago.'
                )
            time.sleep(5)
            updated_job = self.get()
            if updated_job is None:
                raise ValueError(f'Fine-tuning job "{self.display_name}" not found')
            self = updated_job
        return self

    async def await_for_completion(self) -> "SupervisedFineTuningJob":
        """
        Asynchronously poll for job completion.
        """
        while self._proto.state != JobState.COMPLETED:
            if self._proto.state == JobState.FAILED:
                raise ValueError(f'Fine-tuning job "{self.display_name}" failed')
            if self._proto.create_time is not None:
                curr_time = time.time()
                create_time = self._proto.create_time.seconds + self._proto.create_time.nanos / 1e9
                delta_seconds = int(curr_time - create_time)
                minutes = delta_seconds // 60
                seconds = delta_seconds % 60
                logger.debug(
                    f'Fine-tuning job "{self.display_name}" is in state {SyncJobState.Name(self._proto.state)}. Job was created {generate_time_str(minutes, seconds)} ago.'
                )
            await asyncio.sleep(5)
            updated_job = self.get()
            if updated_job is None:
                raise ValueError(f'Fine-tuning job "{self.display_name}" not found')
            self = updated_job
        return self

    def get(self) -> Optional["SupervisedFineTuningJob"]:
        job = self._gateway.get_supervised_fine_tuning_job_sync(self.name)
        if job is None:
            return None
        return SupervisedFineTuningJob(
            llm=self.llm,
            proto=job,
            dataset_or_id=self.dataset_or_id,
            api_key=self._api_key,
        )


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: gateway/mcp_server.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'gateway/mcp_server.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from . import options_pb2 as gateway_dot_options__pb2
from . import status_pb2 as gateway_dot_status__pb2
from google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from google.api import resource_pb2 as google_dot_api_dot_resource__pb2
from google.api import visibility_pb2 as google_dot_api_dot_visibility__pb2
from google.protobuf import field_mask_pb2 as google_dot_protobuf_dot_field__mask__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18gateway/mcp_server.proto\x12\x07gateway\x1a\x1b\x62uf/validate/validate.proto\x1a\x15gateway/options.proto\x1a\x14gateway/status.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x19google/api/resource.proto\x1a\x1bgoogle/api/visibility.proto\x1a google/protobuf/field_mask.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\xd7\x07\n\tMcpServer\x12\x13\n\x04name\x18\x01 \x01(\tB\x05\xe2\x41\x02\x03\x05\x12!\n\x0c\x64isplay_name\x18\x02 \x01(\tB\x0b\xe2\x41\x01\x01\xbaH\x04r\x02\x18?\x12!\n\x0b\x64\x65scription\x18\x03 \x01(\tB\x0c\xe2\x41\x01\x01\xbaH\x05r\x03\x18\xa0\x1f\x12\x36\n\x0b\x63reate_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x05\xe2\x41\x02\x03\x05\x12\x35\n\x0bupdate_time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x04\xe2\x41\x01\x03\x12\x1a\n\x0c\x65ndpoint_url\x18\x06 \x01(\tB\x04\xe2\x41\x01\x01\x12\x1c\n\x0e\x61pi_key_secret\x18\x07 \x01(\tB\x04\xe2\x41\x01\x01\x12\x17\n\tsimulated\x18\x08 \x01(\x08\x42\x04\xe2\x41\x01\x01\x12.\n\x05state\x18\n \x01(\x0e\x32\x18.gateway.McpServer.StateB\x05\xe2\x41\x02\x03\x05\x12&\n\x06status\x18\x0b \x01(\x0b\x32\x0f.gateway.StatusB\x05\xe2\x41\x02\x03\x05\x12\x1b\n\rremote_hosted\x18\x0c \x01(\x08\x42\x04\xe2\x41\x01\x01\x12>\n\x0b\x61nnotations\x18\r \x03(\x0b\x32#.gateway.McpServer.AnnotationsEntryB\x04\xe2\x41\x01\x01\x12\x14\n\x06public\x18\x0e \x01(\x08\x42\x04\xe2\x41\x01\x01\x12H\n\x13\x61uthentication_type\x18\x0f \x01(\x0e\x32%.gateway.McpServer.AuthenticationTypeB\x04\xe2\x41\x01\x01\x12,\n\x08\x66\x65\x61tured\x18\x11 \x01(\x08\x42\x1a\xe2\x41\x01\x01\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x1a\x32\n\x10\x41nnotationsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"n\n\x12\x41uthenticationType\x12#\n\x1f\x41UTHENTICATION_TYPE_UNSPECIFIED\x10\x00\x12\x08\n\x04OPEN\x10\x01\x12\x0b\n\x07\x41PI_KEY\x10\x02\x12\n\n\x06OAUTH2\x10\x03\x12\x10\n\x0c\x42\x45\x41RER_TOKEN\x10\x04\"R\n\x05State\x12\x15\n\x11STATE_UNSPECIFIED\x10\x00\x12\x0c\n\x08\x43REATING\x10\x01\x12\n\n\x06\x41\x43TIVE\x10\x02\x12\n\n\x06\x46\x41ILED\x10\x03\x12\x0c\n\x08\x44\x45LETING\x10\x04:r\xea\x41\x62\n\x1a\x61pi.fireworks.ai/McpServer\x12-accounts/{AccountId}/mcpServers/{McpServerId}*\nmcpServers2\tmcpServer\x82\xf1\x04\t\n\x07\x41\x63\x63ount\"y\n\x16\x43reateMcpServerRequest\x12\x14\n\x06parent\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12,\n\nmcp_server\x18\x02 \x01(\x0b\x32\x12.gateway.McpServerB\x04\xe2\x41\x01\x02\x12\x1b\n\rmcp_server_id\x18\x03 \x01(\tB\x04\xe2\x41\x01\x02\"^\n\x13GetMcpServerRequest\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12\x33\n\tread_mask\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.FieldMaskB\x04\xe2\x41\x01\x01\"\xc3\x01\n\x15ListMcpServersRequest\x12\x14\n\x06parent\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12\x17\n\tpage_size\x18\x02 \x01(\x05\x42\x04\xe2\x41\x01\x01\x12\x18\n\npage_token\x18\x03 \x01(\tB\x04\xe2\x41\x01\x01\x12\x14\n\x06\x66ilter\x18\x04 \x01(\tB\x04\xe2\x41\x01\x01\x12\x16\n\x08order_by\x18\x05 \x01(\tB\x04\xe2\x41\x01\x01\x12\x33\n\tread_mask\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.FieldMaskB\x04\xe2\x41\x01\x01\"n\n\x16ListMcpServersResponse\x12\'\n\x0bmcp_servers\x18\x01 \x03(\x0b\x32\x12.gateway.McpServer\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t\x12\x12\n\ntotal_size\x18\x03 \x01(\x05\"}\n\x16UpdateMcpServerRequest\x12,\n\nmcp_server\x18\x01 \x01(\x0b\x32\x12.gateway.McpServerB\x04\xe2\x41\x01\x02\x12\x35\n\x0bupdate_mask\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.FieldMaskB\x04\xe2\x41\x01\x01\",\n\x16\x44\x65leteMcpServerRequest\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x42\x43ZAgithub.com/fw-ai/fireworks/control_plane/protos/generated/gatewayb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'gateway.mcp_server_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'ZAgithub.com/fw-ai/fireworks/control_plane/protos/generated/gateway'
  _globals['_MCPSERVER_ANNOTATIONSENTRY']._loaded_options = None
  _globals['_MCPSERVER_ANNOTATIONSENTRY']._serialized_options = b'8\001'
  _globals['_MCPSERVER'].fields_by_name['name']._loaded_options = None
  _globals['_MCPSERVER'].fields_by_name['name']._serialized_options = b'\342A\002\003\005'
  _globals['_MCPSERVER'].fields_by_name['display_name']._loaded_options = None
  _globals['_MCPSERVER'].fields_by_name['display_name']._serialized_options = b'\342A\001\001\272H\004r\002\030?'
  _globals['_MCPSERVER'].fields_by_name['description']._loaded_options = None
  _globals['_MCPSERVER'].fields_by_name['description']._serialized_options = b'\342A\001\001\272H\005r\003\030\240\037'
  _globals['_MCPSERVER'].fields_by_name['create_time']._loaded_options = None
  _globals['_MCPSERVER'].fields_by_name['create_time']._serialized_options = b'\342A\002\003\005'
  _globals['_MCPSERVER'].fields_by_name['update_time']._loaded_options = None
  _globals['_MCPSERVER'].fields_by_name['update_time']._serialized_options = b'\342A\001\003'
  _globals['_MCPSERVER'].fields_by_name['endpoint_url']._loaded_options = None
  _globals['_MCPSERVER'].fields_by_name['endpoint_url']._serialized_options = b'\342A\001\001'
  _globals['_MCPSERVER'].fields_by_name['api_key_secret']._loaded_options = None
  _globals['_MCPSERVER'].fields_by_name['api_key_secret']._serialized_options = b'\342A\001\001'
  _globals['_MCPSERVER'].fields_by_name['simulated']._loaded_options = None
  _globals['_MCPSERVER'].fields_by_name['simulated']._serialized_options = b'\342A\001\001'
  _globals['_MCPSERVER'].fields_by_name['state']._loaded_options = None
  _globals['_MCPSERVER'].fields_by_name['state']._serialized_options = b'\342A\002\003\005'
  _globals['_MCPSERVER'].fields_by_name['status']._loaded_options = None
  _globals['_MCPSERVER'].fields_by_name['status']._serialized_options = b'\342A\002\003\005'
  _globals['_MCPSERVER'].fields_by_name['remote_hosted']._loaded_options = None
  _globals['_MCPSERVER'].fields_by_name['remote_hosted']._serialized_options = b'\342A\001\001'
  _globals['_MCPSERVER'].fields_by_name['annotations']._loaded_options = None
  _globals['_MCPSERVER'].fields_by_name['annotations']._serialized_options = b'\342A\001\001'
  _globals['_MCPSERVER'].fields_by_name['public']._loaded_options = None
  _globals['_MCPSERVER'].fields_by_name['public']._serialized_options = b'\342A\001\001'
  _globals['_MCPSERVER'].fields_by_name['authentication_type']._loaded_options = None
  _globals['_MCPSERVER'].fields_by_name['authentication_type']._serialized_options = b'\342A\001\001'
  _globals['_MCPSERVER'].fields_by_name['featured']._loaded_options = None
  _globals['_MCPSERVER'].fields_by_name['featured']._serialized_options = b'\342A\001\001\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_MCPSERVER']._loaded_options = None
  _globals['_MCPSERVER']._serialized_options = b'\352Ab\n\032api.fireworks.ai/McpServer\022-accounts/{AccountId}/mcpServers/{McpServerId}*\nmcpServers2\tmcpServer\202\361\004\t\n\007Account'
  _globals['_CREATEMCPSERVERREQUEST'].fields_by_name['parent']._loaded_options = None
  _globals['_CREATEMCPSERVERREQUEST'].fields_by_name['parent']._serialized_options = b'\342A\001\002'
  _globals['_CREATEMCPSERVERREQUEST'].fields_by_name['mcp_server']._loaded_options = None
  _globals['_CREATEMCPSERVERREQUEST'].fields_by_name['mcp_server']._serialized_options = b'\342A\001\002'
  _globals['_CREATEMCPSERVERREQUEST'].fields_by_name['mcp_server_id']._loaded_options = None
  _globals['_CREATEMCPSERVERREQUEST'].fields_by_name['mcp_server_id']._serialized_options = b'\342A\001\002'
  _globals['_GETMCPSERVERREQUEST'].fields_by_name['name']._loaded_options = None
  _globals['_GETMCPSERVERREQUEST'].fields_by_name['name']._serialized_options = b'\342A\001\002'
  _globals['_GETMCPSERVERREQUEST'].fields_by_name['read_mask']._loaded_options = None
  _globals['_GETMCPSERVERREQUEST'].fields_by_name['read_mask']._serialized_options = b'\342A\001\001'
  _globals['_LISTMCPSERVERSREQUEST'].fields_by_name['parent']._loaded_options = None
  _globals['_LISTMCPSERVERSREQUEST'].fields_by_name['parent']._serialized_options = b'\342A\001\002'
  _globals['_LISTMCPSERVERSREQUEST'].fields_by_name['page_size']._loaded_options = None
  _globals['_LISTMCPSERVERSREQUEST'].fields_by_name['page_size']._serialized_options = b'\342A\001\001'
  _globals['_LISTMCPSERVERSREQUEST'].fields_by_name['page_token']._loaded_options = None
  _globals['_LISTMCPSERVERSREQUEST'].fields_by_name['page_token']._serialized_options = b'\342A\001\001'
  _globals['_LISTMCPSERVERSREQUEST'].fields_by_name['filter']._loaded_options = None
  _globals['_LISTMCPSERVERSREQUEST'].fields_by_name['filter']._serialized_options = b'\342A\001\001'
  _globals['_LISTMCPSERVERSREQUEST'].fields_by_name['order_by']._loaded_options = None
  _globals['_LISTMCPSERVERSREQUEST'].fields_by_name['order_by']._serialized_options = b'\342A\001\001'
  _globals['_LISTMCPSERVERSREQUEST'].fields_by_name['read_mask']._loaded_options = None
  _globals['_LISTMCPSERVERSREQUEST'].fields_by_name['read_mask']._serialized_options = b'\342A\001\001'
  _globals['_UPDATEMCPSERVERREQUEST'].fields_by_name['mcp_server']._loaded_options = None
  _globals['_UPDATEMCPSERVERREQUEST'].fields_by_name['mcp_server']._serialized_options = b'\342A\001\002'
  _globals['_UPDATEMCPSERVERREQUEST'].fields_by_name['update_mask']._loaded_options = None
  _globals['_UPDATEMCPSERVERREQUEST'].fields_by_name['update_mask']._serialized_options = b'\342A\001\001'
  _globals['_DELETEMCPSERVERREQUEST'].fields_by_name['name']._loaded_options = None
  _globals['_DELETEMCPSERVERREQUEST'].fields_by_name['name']._serialized_options = b'\342A\001\002'
  _globals['_MCPSERVER']._serialized_start=268
  _globals['_MCPSERVER']._serialized_end=1251
  _globals['_MCPSERVER_ANNOTATIONSENTRY']._serialized_start=889
  _globals['_MCPSERVER_ANNOTATIONSENTRY']._serialized_end=939
  _globals['_MCPSERVER_AUTHENTICATIONTYPE']._serialized_start=941
  _globals['_MCPSERVER_AUTHENTICATIONTYPE']._serialized_end=1051
  _globals['_MCPSERVER_STATE']._serialized_start=1053
  _globals['_MCPSERVER_STATE']._serialized_end=1135
  _globals['_CREATEMCPSERVERREQUEST']._serialized_start=1253
  _globals['_CREATEMCPSERVERREQUEST']._serialized_end=1374
  _globals['_GETMCPSERVERREQUEST']._serialized_start=1376
  _globals['_GETMCPSERVERREQUEST']._serialized_end=1470
  _globals['_LISTMCPSERVERSREQUEST']._serialized_start=1473
  _globals['_LISTMCPSERVERSREQUEST']._serialized_end=1668
  _globals['_LISTMCPSERVERSRESPONSE']._serialized_start=1670
  _globals['_LISTMCPSERVERSRESPONSE']._serialized_end=1780
  _globals['_UPDATEMCPSERVERREQUEST']._serialized_start=1782
  _globals['_UPDATEMCPSERVERREQUEST']._serialized_end=1907
  _globals['_DELETEMCPSERVERREQUEST']._serialized_start=1909
  _globals['_DELETEMCPSERVERREQUEST']._serialized_end=1953
# @@protoc_insertion_point(module_scope)



import time
from datetime import datetime
from typing import Optional, Union, TYPE_CHECKING
from fireworks.gateway import Gateway
from fireworks.dataset import Dataset
from fireworks.control_plane.generated.protos.gateway import (
    JobState,
)
from fireworks.control_plane.generated.protos_grpcio.gateway.batch_inference_job_pb2 import (
    BatchInferenceJob as SyncBatchInferenceJob,
    CreateBatchInferenceJobRequest as SyncCreateBatchInferenceJobRequest,
    ListBatchInferenceJobsRequest as SyncListBatchInferenceJobsRequest,
    InferenceParameters as SyncInferenceParameters,
)
from fireworks.control_plane.generated.protos_grpcio.gateway.status_pb2 import (
    JobState as SyncJobState,
)
from fireworks._logger import logger
from google.protobuf.descriptor import FieldDescriptor


def _enum_to_string(field, value):
    """Convert protobuf enum value to string name.
    
    Args:
        field: The protobuf field descriptor
        value: The enum value (integer)
        
    Returns:
        String representation of the enum value
    """
    if field.type == FieldDescriptor.TYPE_ENUM:
        try:
            # Get the enum descriptor and find the value name
            enum_value = field.enum_type.values_by_number.get(value)
            if enum_value:
                return enum_value.name
        except Exception:
            pass
    return str(value)


class BatchInferenceJob:
    """
    Wrapper around proto for a batch inference job in Fireworks. Provides
    convenient methods for CRUD operations: create, get, list, and delete.
    
    Similar to firectl's batch inference job commands (bij).
    """

    def __init__(self):
        """
        Stateless wrapper for batch inference job operations.
        All operations are performed via class/static methods.
        """
        pass

    @staticmethod
    def _get_job_id_from_name(name: str) -> str:
        """Extract job ID from full resource name."""
        return name.split("/")[-1] if name else ""

    @staticmethod
    def _normalize_job_name(name: str, account: str) -> str:
        """Ensure job name is in full resource format."""
        if not name.startswith("accounts/"):
            return f"accounts/{account}/batchInferenceJobs/{name}"
        return name

    @staticmethod
    def get(job_id: str, account: str, api_key: Optional[str] = None) -> Optional[SyncBatchInferenceJob]:
        """
        Get a batch inference job by its ID.
        Equivalent to: firectl get bij <job-id>
        
        Args:
            job_id: The job ID (e.g., "test-job-123") or full resource name
            account: Account ID
            api_key: The API key to use
            
        Returns:
            BatchInferenceJob proto if found, None otherwise
        """
        gateway = Gateway(api_key=api_key)
        
        name = BatchInferenceJob._normalize_job_name(job_id, account)
        
        proto = gateway.get_batch_inference_job_sync(name)
        if proto is None:
            return None
            
        return proto

    @staticmethod
    def list(
        account: str,
        api_key: Optional[str] = None,
        page_size: int = 50
    ) -> list[SyncBatchInferenceJob]:
        """
        List batch inference jobs in an account.
        Equivalent to: firectl list bij
        
        Args:
            account: Account ID
            api_key: The API key to use
            page_size: Number of jobs to return per page
            
        Returns:
            List of BatchInferenceJob protos
        """
        gateway = Gateway(api_key=api_key)
        
        request = SyncListBatchInferenceJobsRequest()
        request.parent = f"accounts/{account}"
        request.page_size = page_size
        
        response = gateway._sync_stub.ListBatchInferenceJobs(request)
        
        return list(response.batch_inference_jobs)

    @staticmethod
    def create(
        model: str,
        input_dataset_id: str,
        output_dataset_id: Optional[str] = None,
        job_id: Optional[str] = None,
        display_name: Optional[str] = None,
        inference_parameters: Optional[dict] = None,
        api_key: Optional[str] = None,
    ) -> SyncBatchInferenceJob:
        """
        Create a new batch inference job.
        Equivalent to: firectl create bij --model <model> --input-dataset-id <dataset>
        
        Args:
            model: The model to use for inference
            input_dataset_id: The input dataset ID
            output_dataset_id: The output dataset ID (optional)
            job_id: The job ID (optional, will be auto-generated if not provided)
            display_name: Display name for the job (optional)
            inference_parameters: Dict of inference parameters like max_tokens, temperature, etc.
            api_key: The API key to use
            
        Returns:
            BatchInferenceJob proto
        """
        gateway = Gateway(api_key=api_key)
        account_id = gateway.account_id()
        
        # Normalize resource names
        if not model.startswith("accounts/"):
            model = f"accounts/{account_id}/models/{model}"
        if not input_dataset_id.startswith("accounts/"):
            input_dataset_id = f"accounts/{account_id}/datasets/{input_dataset_id}"
        if output_dataset_id and not output_dataset_id.startswith("accounts/"):
            output_dataset_id = f"accounts/{account_id}/datasets/{output_dataset_id}"
        
        # Create the batch inference job proto
        job_proto = SyncBatchInferenceJob()
        job_proto.model = model
        job_proto.input_dataset_id = input_dataset_id
        if output_dataset_id:
            job_proto.output_dataset_id = output_dataset_id
        if display_name:
            job_proto.display_name = display_name
            
        # Set inference parameters if provided
        if inference_parameters:
            params = SyncInferenceParameters()
            
            if "max_tokens" in inference_parameters:
                params.max_tokens = inference_parameters["max_tokens"]
            if "temperature" in inference_parameters:
                params.temperature = inference_parameters["temperature"]
            if "top_p" in inference_parameters:
                params.top_p = inference_parameters["top_p"]
            if "top_k" in inference_parameters:
                params.top_k = inference_parameters["top_k"]
            if "n" in inference_parameters:
                params.n = inference_parameters["n"]
            if "extra_body" in inference_parameters:
                params.extra_body = inference_parameters["extra_body"]
                
            job_proto.inference_parameters.CopyFrom(params)
        
        # Create the request
        request = SyncCreateBatchInferenceJobRequest()
        request.batch_inference_job.CopyFrom(job_proto)
        if job_id:
            request.batch_inference_job_id = job_id
        
        # Create the job
        created_proto = gateway.create_batch_inference_job_sync(request)
        return created_proto


    @staticmethod
    def delete(job_id: str, account: str, api_key: Optional[str] = None) -> None:
        """
        Delete a batch inference job by ID.
        Equivalent to: firectl delete bij <job-id>
        
        Args:
            job_id: The job ID or full resource name
            account: Account ID
            api_key: The API key to use
        """
        gateway = Gateway(api_key=api_key)
        
        name = BatchInferenceJob._normalize_job_name(job_id, account)
        
        gateway.delete_batch_inference_job_sync(name)

    @staticmethod
    def to_dict(proto: SyncBatchInferenceJob) -> dict:
        """Convenience method to convert a batch inference job proto to a friendly representation."""
        if not proto:
            return {}
        
        result = {}
        for field, value in proto.ListFields():
            if field.name in ['name', 'display_name', 'model', 'input_dataset_id', 'output_dataset_id', 'created_by']:
                result[field.name] = str(value)
            elif field.type == FieldDescriptor.TYPE_ENUM:
                # Convert any enum field to readable string name
                result[field.name] = _enum_to_string(field, value)
            elif field.name in ['create_time', 'update_time']:
                # Convert timestamp to readable format in UTC
                if hasattr(value, 'seconds') and hasattr(value, 'nanos'):
                    timestamp = value.seconds + value.nanos / 1e9
                    dt = datetime.utcfromtimestamp(timestamp)
                    result[field.name] = dt.strftime('%Y-%m-%d %H:%M:%S UTC')
                else:
                    result[field.name] = str(value)
            elif field.name == 'inference_parameters':
                params = {}
                for param_field, param_value in value.ListFields():
                    params[param_field.name] = param_value
                result['inference_parameters'] = params
            elif field.name == 'status' and hasattr(value, 'message'):
                result['status_message'] = value.message
        
        return result



from ..buf.validate import validate_pb2 as _validate_pb2
from . import deployment_pb2 as _deployment_pb2
from . import options_pb2 as _options_pb2
from . import status_pb2 as _status_pb2
from google.api import field_behavior_pb2 as _field_behavior_pb2
from google.api import resource_pb2 as _resource_pb2
from google.api import visibility_pb2 as _visibility_pb2
from google.protobuf import field_mask_pb2 as _field_mask_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Workload(_message.Message):
    __slots__ = ("name", "display_name", "description", "create_time", "update_time", "expire_time", "purge_time", "delete_time", "created_by", "state", "status", "annotations", "placement", "min_replica_count", "max_replica_count", "desired_replica_count", "autoscaling_policy", "workload_shape", "disable_accounting", "for_training")
    class State(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        STATE_UNSPECIFIED: _ClassVar[Workload.State]
        CREATING: _ClassVar[Workload.State]
        READY: _ClassVar[Workload.State]
        DELETING: _ClassVar[Workload.State]
        FAILED: _ClassVar[Workload.State]
        UPDATING: _ClassVar[Workload.State]
        DELETED: _ClassVar[Workload.State]
    STATE_UNSPECIFIED: Workload.State
    CREATING: Workload.State
    READY: Workload.State
    DELETING: Workload.State
    FAILED: Workload.State
    UPDATING: Workload.State
    DELETED: Workload.State
    class AnnotationsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    NAME_FIELD_NUMBER: _ClassVar[int]
    DISPLAY_NAME_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    CREATE_TIME_FIELD_NUMBER: _ClassVar[int]
    UPDATE_TIME_FIELD_NUMBER: _ClassVar[int]
    EXPIRE_TIME_FIELD_NUMBER: _ClassVar[int]
    PURGE_TIME_FIELD_NUMBER: _ClassVar[int]
    DELETE_TIME_FIELD_NUMBER: _ClassVar[int]
    CREATED_BY_FIELD_NUMBER: _ClassVar[int]
    STATE_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    ANNOTATIONS_FIELD_NUMBER: _ClassVar[int]
    PLACEMENT_FIELD_NUMBER: _ClassVar[int]
    MIN_REPLICA_COUNT_FIELD_NUMBER: _ClassVar[int]
    MAX_REPLICA_COUNT_FIELD_NUMBER: _ClassVar[int]
    DESIRED_REPLICA_COUNT_FIELD_NUMBER: _ClassVar[int]
    AUTOSCALING_POLICY_FIELD_NUMBER: _ClassVar[int]
    WORKLOAD_SHAPE_FIELD_NUMBER: _ClassVar[int]
    DISABLE_ACCOUNTING_FIELD_NUMBER: _ClassVar[int]
    FOR_TRAINING_FIELD_NUMBER: _ClassVar[int]
    name: str
    display_name: str
    description: str
    create_time: _timestamp_pb2.Timestamp
    update_time: _timestamp_pb2.Timestamp
    expire_time: _timestamp_pb2.Timestamp
    purge_time: _timestamp_pb2.Timestamp
    delete_time: _timestamp_pb2.Timestamp
    created_by: str
    state: Workload.State
    status: _status_pb2.Status
    annotations: _containers.ScalarMap[str, str]
    placement: _deployment_pb2.Placement
    min_replica_count: int
    max_replica_count: int
    desired_replica_count: int
    autoscaling_policy: _deployment_pb2.AutoscalingPolicy
    workload_shape: str
    disable_accounting: bool
    for_training: bool
    def __init__(self, name: _Optional[str] = ..., display_name: _Optional[str] = ..., description: _Optional[str] = ..., create_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., update_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., expire_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., purge_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., delete_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., created_by: _Optional[str] = ..., state: _Optional[_Union[Workload.State, str]] = ..., status: _Optional[_Union[_status_pb2.Status, _Mapping]] = ..., annotations: _Optional[_Mapping[str, str]] = ..., placement: _Optional[_Union[_deployment_pb2.Placement, _Mapping]] = ..., min_replica_count: _Optional[int] = ..., max_replica_count: _Optional[int] = ..., desired_replica_count: _Optional[int] = ..., autoscaling_policy: _Optional[_Union[_deployment_pb2.AutoscalingPolicy, _Mapping]] = ..., workload_shape: _Optional[str] = ..., disable_accounting: bool = ..., for_training: bool = ...) -> None: ...

class CreateWorkloadRequest(_message.Message):
    __slots__ = ("parent", "workload", "workload_id")
    PARENT_FIELD_NUMBER: _ClassVar[int]
    WORKLOAD_FIELD_NUMBER: _ClassVar[int]
    WORKLOAD_ID_FIELD_NUMBER: _ClassVar[int]
    parent: str
    workload: Workload
    workload_id: str
    def __init__(self, parent: _Optional[str] = ..., workload: _Optional[_Union[Workload, _Mapping]] = ..., workload_id: _Optional[str] = ...) -> None: ...

class GetWorkloadRequest(_message.Message):
    __slots__ = ("name", "read_mask")
    NAME_FIELD_NUMBER: _ClassVar[int]
    READ_MASK_FIELD_NUMBER: _ClassVar[int]
    name: str
    read_mask: _field_mask_pb2.FieldMask
    def __init__(self, name: _Optional[str] = ..., read_mask: _Optional[_Union[_field_mask_pb2.FieldMask, _Mapping]] = ...) -> None: ...

class ListWorkloadsRequest(_message.Message):
    __slots__ = ("parent", "page_size", "page_token", "filter", "order_by", "show_deleted", "show_internal", "read_mask")
    PARENT_FIELD_NUMBER: _ClassVar[int]
    PAGE_SIZE_FIELD_NUMBER: _ClassVar[int]
    PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    FILTER_FIELD_NUMBER: _ClassVar[int]
    ORDER_BY_FIELD_NUMBER: _ClassVar[int]
    SHOW_DELETED_FIELD_NUMBER: _ClassVar[int]
    SHOW_INTERNAL_FIELD_NUMBER: _ClassVar[int]
    READ_MASK_FIELD_NUMBER: _ClassVar[int]
    parent: str
    page_size: int
    page_token: str
    filter: str
    order_by: str
    show_deleted: bool
    show_internal: bool
    read_mask: _field_mask_pb2.FieldMask
    def __init__(self, parent: _Optional[str] = ..., page_size: _Optional[int] = ..., page_token: _Optional[str] = ..., filter: _Optional[str] = ..., order_by: _Optional[str] = ..., show_deleted: bool = ..., show_internal: bool = ..., read_mask: _Optional[_Union[_field_mask_pb2.FieldMask, _Mapping]] = ...) -> None: ...

class ListWorkloadsResponse(_message.Message):
    __slots__ = ("workloads", "next_page_token", "total_size")
    WORKLOADS_FIELD_NUMBER: _ClassVar[int]
    NEXT_PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    TOTAL_SIZE_FIELD_NUMBER: _ClassVar[int]
    workloads: _containers.RepeatedCompositeFieldContainer[Workload]
    next_page_token: str
    total_size: int
    def __init__(self, workloads: _Optional[_Iterable[_Union[Workload, _Mapping]]] = ..., next_page_token: _Optional[str] = ..., total_size: _Optional[int] = ...) -> None: ...

class UpdateWorkloadRequest(_message.Message):
    __slots__ = ("workload", "update_mask")
    WORKLOAD_FIELD_NUMBER: _ClassVar[int]
    UPDATE_MASK_FIELD_NUMBER: _ClassVar[int]
    workload: Workload
    update_mask: _field_mask_pb2.FieldMask
    def __init__(self, workload: _Optional[_Union[Workload, _Mapping]] = ..., update_mask: _Optional[_Union[_field_mask_pb2.FieldMask, _Mapping]] = ...) -> None: ...

class DeleteWorkloadRequest(_message.Message):
    __slots__ = ("name", "hard", "ignore_checks")
    NAME_FIELD_NUMBER: _ClassVar[int]
    HARD_FIELD_NUMBER: _ClassVar[int]
    IGNORE_CHECKS_FIELD_NUMBER: _ClassVar[int]
    name: str
    hard: bool
    ignore_checks: bool
    def __init__(self, name: _Optional[str] = ..., hard: bool = ..., ignore_checks: bool = ...) -> None: ...



from datetime import timedelta
import inspect
import os
import time
from uuid import uuid4

from fireworks._const import FIREWORKS_API_BASE_URL
from fireworks.evaluator.evaluator import Evaluator
from fireworks.llm._types import WandbConfigParam
from fireworks.reinforcement_fine_tuning_job.reinforcement_fine_tuning_job import ReinforcementFineTuningJob
from fireworks.client.api_client import FireworksClient
from functools import cache as sync_cache
from google.protobuf.field_mask_pb2 import FieldMask as SyncFieldMask
from typing import (
    Any,
    Callable,
    List,
    Optional,
    Union,
    TYPE_CHECKING,
)
from fireworks.dataset import Dataset
from fireworks.gateway import Gateway
from google.protobuf.duration_pb2 import Duration
from fireworks.control_plane.generated.protos_grpcio.gateway.deployment_pb2 import (
    Deployment as SyncDeployment,
    AutoscalingPolicy as SyncAutoscalingPolicy,
    AcceleratorType as SyncAcceleratorType,
    Region as SyncRegion,
    MultiRegion as SyncMultiRegion,
    DirectRouteType as SyncDirectRouteType,
    AutoTune as SyncAutoTune,
)
from fireworks.control_plane.generated.protos_grpcio.gateway.model_pb2 import (
    Model as SyncModel,
)
from fireworks.control_plane.generated.protos_grpcio.gateway.deployed_model_pb2 import (
    CreateDeployedModelRequest as SyncCreateDeployedModelRequest,
    DeployedModel as SyncDeployedModel,
    ListDeployedModelsRequest as SyncListDeployedModelsRequest,
)
from fireworks.control_plane.generated.protos_grpcio.gateway.training_pb2 import (
    BaseTrainingConfig as SyncBaseTrainingConfig,
)
from fireworks.control_plane.generated.protos.gateway import (
    AcceleratorType as AcceleratorTypeEnum,
    AutoTune,
    AutoscalingPolicy,
    BaseTrainingConfig,
    Deployment,
    DeploymentState,
    InferenceParameters,
)
from fireworks._util import generate_model_resource_name, is_valid_resource_name
import sysconfig
from fireworks._literals import (
    DeploymentStrategyLiteral,
    DeploymentTypeLiteral,
    DirectRouteTypeLiteral,
    MultiRegionLiteral,
    PrecisionLiteral,
    RegionLiteral,
    AcceleratorTypeLiteral,
    WeightPrecisionLiteral,
)
from fireworks._logger import log_execution_time, logger
from fireworks.control_plane.generated.protos_grpcio.gateway.supervised_fine_tuning_job_pb2 import (
    SupervisedFineTuningJob as SyncSupervisedFineTuningJob,
)
from fireworks.control_plane.generated.protos_grpcio.gateway.reinforcement_fine_tuning_job_pb2 import (
    ReinforcementFineTuningJob as SyncReinforcementFineTuningJob,
)
from fireworks.control_plane.generated.protos_grpcio.gateway.training_pb2 import (
    BaseTrainingConfig as SyncBaseTrainingConfig,
    WeightPrecision,
)
from fireworks.control_plane.generated.protos_grpcio.gateway.wandb_pb2 import (
    WandbConfig as SyncWandbConfig,
)
from fireworks.control_plane.generated.protos_grpcio.gateway.batch_inference_job_pb2 import (
    InferenceParameters as SyncInferenceParameters,
)
from fireworks.llm.chat_completion import ChatCompletion, Chat
from fireworks.llm.completion import Completion
from fireworks.llm.metrics import Metrics
from fireworks.llm.responses import Responses

# Type checking imports to avoid circular imports
if TYPE_CHECKING:
    from fireworks.supervised_fine_tuning_job import SupervisedFineTuningJob

DEFAULT_MAX_RETRIES = 10
DEFAULT_DELAY = 0.5


class LLM:
    def __init__(
        self,
        model: str,
        deployment_type: DeploymentTypeLiteral,
        id: Optional[str] = None,
        deployment_display_name: Optional[str] = None,
        base_id: Optional[str] = None,
        api_key: Optional[str] = None,
        direct_route_api_key: Optional[str] = None,
        base_url: str = f"{FIREWORKS_API_BASE_URL}/inference/v1",
        max_retries: int = DEFAULT_MAX_RETRIES,
        request_timeout: int = 600,
        scale_up_window: timedelta = timedelta(seconds=1),
        scale_down_window: timedelta = timedelta(minutes=1),
        scale_to_zero_window: timedelta = timedelta(minutes=5),
        enable_metrics: bool = False,
        accelerator_type: Optional[AcceleratorTypeLiteral] = None,
        region: Optional[RegionLiteral] = None,
        multi_region: Optional[MultiRegionLiteral] = None,
        description: Optional[str] = None,
        annotations: Optional[dict[str, str]] = None,
        min_replica_count: Optional[int] = None,
        max_replica_count: Optional[int] = None,
        replica_count: Optional[int] = None,
        accelerator_count: Optional[int] = None,
        precision: Optional[PrecisionLiteral] = None,
        world_size: Optional[int] = None,
        generator_count: Optional[int] = None,
        disaggregated_prefill_count: Optional[int] = None,
        disaggregated_prefill_world_size: Optional[int] = None,
        max_batch_size: Optional[int] = None,
        cluster: Optional[str] = None,
        enable_addons: Optional[bool] = None,
        draft_token_count: Optional[int] = None,
        draft_model: Optional[str] = None,
        ngram_speculation_length: Optional[int] = None,
        max_peft_batch_size: Optional[int] = None,
        kv_cache_memory_pct: Optional[int] = None,
        enable_session_affinity: Optional[bool] = None,
        direct_route_api_keys: Optional[list[str]] = None,
        num_peft_device_cached: Optional[int] = None,
        direct_route_type: Optional[DirectRouteTypeLiteral] = None,
        direct_route_handle: Optional[str] = None,
        long_prompt_optimized: Optional[bool] = None,
        temperature: Optional[float] = None,
        perf_metrics_in_response: Optional[bool] = None,
        max_connections: Optional[int] = None,
        client_kwargs: dict[str, Any] = {},
    ):
        """
        Initialize an LLM object. Performs validation of the model and deployment type.

        Args:
            model: The model to use.
            deployment_type: The deployment type to use. Must be one of
                "serverless", "on-demand", "auto", or "on-demand-lora". For experimentation on
                quality, we recommend using "auto" to default to the most
                cost-effective option. If you plan to run large evaluation jobs or
                have workloads that would benefit from dedicated resources, we
                recommend using "on-demand". Otherwise, you can enforce that you
                only use "serverless" by setting this parameter to "serverless".
            id: The id of the deployment to use. This is required when deployment_type is "on-demand".
            base_id: The base deployment id to use. This is required when deployment_type is "on-demand-lora".
            deployment_display_name: The display name of the deployment to use.
                If not provided, the deployment display name will be generated
                from the filename of the caller where this LLM was instantiated.
            api_key: The API key to use.
            direct_route_api_key: The API key to use for direct route.
            base_url: The base URL to use.
            accelerator_type: The accelerator type to use.
            scale_up_window: The scale up window to use.
            scale_down_window: The scale down window to use.
            scale_to_zero_window: The scale to zero window to use.
            enable_metrics: Whether to enable metrics. Only records time to last token for non-streaming requests.
            max_retries: The maximum number of retries to use.
            max_connections: The maximum number of concurrent connections to configure for the FireworksClient.
            client_kwargs: Additional keyword arguments to pass to the FireworksClient.
            For more information on parameters, see the [API Reference](https://docs.fireworks.ai/api-reference/post-chatcompletions)
        """
        if not model:
            raise ValueError("model is required")
        if deployment_type is None:
            raise ValueError('deployment_type is required - must be one of "serverless", "on-demand", or "auto"')
        self._direct_route_api_key = direct_route_api_key
        self._client = FireworksClient(
            request_timeout=request_timeout,
            api_key=direct_route_api_key if self.direct_route_api_key else api_key,
            base_url=base_url,
            max_connections=max_connections,
            **client_kwargs,
        )
        self._id = id
        self._gateway = Gateway(api_key=api_key)
        self._deployment_display_name = deployment_display_name
        self._direct_route_api_key = direct_route_api_key
        self._ran_setup = False
        self._model = model
        if base_id is not None and not self.is_peft_addon():
            raise ValueError("base_id is only for PEFT addons. Use id instead for specifying an existing deployment.")
        self._base_id = base_id
        self.chat = Chat(self, self.model)
        self.completions = Completion(self)
        self.max_retries = max_retries
        self.responses = Responses(self)
        self.deployment_type: DeploymentTypeLiteral = deployment_type
        self.deployment_strategy = self._compute_deployment_strategy()
        if self.deployment_strategy == "on-demand":
            if id is None:
                raise ValueError("LLM(id=...) must be provided when deployment_strategy is on-demand")
            if id == "":
                raise ValueError("LLM(id=...) must not be empty when deployment_strategy is on-demand")
        if self.deployment_strategy == "on-demand-lora":
            if id is not None:
                raise ValueError(
                    "LLM(id=...) must not be provided when deployment_strategy is on-demand-lora. Fireworks automatically generates an ID when loading a LoRA."
                )
            if base_id is None:
                raise ValueError("LLM(base_id=...) must be provided when deployment_strategy is on-demand-lora")
            if base_id == "":
                raise ValueError("LLM(base_id=...) must not be empty when deployment_strategy is on-demand-lora")
        self.enable_metrics = enable_metrics
        self._metrics = Metrics()

        # This needs to be run in __init__ to ensure we capture deployment name
        # inside of this thread
        self._get_deployment_display_name()

        self._accelerator_type_original: Optional[AcceleratorTypeLiteral] = accelerator_type
        if isinstance(accelerator_type, str):
            self._accelerator_type = AcceleratorTypeEnum.from_string(accelerator_type)
            self._validate_model_for_gpu(self.model, self._accelerator_type)
            self._accelerator_type_sync: Optional[SyncAcceleratorType] = getattr(SyncAcceleratorType, accelerator_type)
        else:
            self._accelerator_type = accelerator_type
            self._accelerator_type_sync = accelerator_type

        # aggressive defaults for experimentation to save on cost
        self._autoscaling_policy: AutoscalingPolicy = AutoscalingPolicy(
            scale_up_window=scale_up_window,
            scale_down_window=scale_down_window,
            scale_to_zero_window=scale_to_zero_window,
        )
        self._autoscaling_policy_sync = SyncAutoscalingPolicy(
            scale_up_window=Duration(seconds=int(scale_up_window.total_seconds())),
            scale_down_window=Duration(seconds=int(scale_down_window.total_seconds())),
            scale_to_zero_window=Duration(seconds=int(scale_to_zero_window.total_seconds())),
        )
        self._region: Optional[RegionLiteral] = region
        self._multi_region: Optional[MultiRegionLiteral] = multi_region
        self._description = description
        self._annotations = annotations
        self._min_replica_count = min_replica_count
        self._max_replica_count = max_replica_count
        self._replica_count = replica_count
        self._accelerator_count = accelerator_count
        self._precision: Optional[PrecisionLiteral] = precision
        self._world_size = world_size
        self._generator_count = generator_count
        self._disaggregated_prefill_count = disaggregated_prefill_count
        self._disaggregated_prefill_world_size = disaggregated_prefill_world_size
        self._max_batch_size = max_batch_size
        self._cluster = cluster
        self._enable_addons = enable_addons
        self._draft_token_count = draft_token_count
        self._draft_model = draft_model
        self._ngram_speculation_length = ngram_speculation_length
        self._max_peft_batch_size = max_peft_batch_size
        self._kv_cache_memory_pct = kv_cache_memory_pct
        self._enable_session_affinity = enable_session_affinity
        self._direct_route_api_keys = direct_route_api_keys
        self._num_peft_device_cached = num_peft_device_cached
        self._direct_route_type: Optional[DirectRouteTypeLiteral] = direct_route_type
        self._direct_route_handle = direct_route_handle
        self._temperature = temperature
        self._perf_metrics_in_response = perf_metrics_in_response
        self._auto_tune = AutoTune()
        self._auto_tune_sync = SyncAutoTune()
        if long_prompt_optimized is not None:
            self._auto_tune.long_prompt_optimized = long_prompt_optimized
            self._auto_tune_sync.long_prompt = long_prompt_optimized

        if not self.is_available_on_serverless() and self.deployment_type == "serverless":
            raise ValueError(
                f"Model {self.model} is not available on serverless, but deployment_type is serverless, please use deployment_type='auto' or 'on-demand'"
            )

    @property
    def direct_route_api_key(self) -> Optional[str]:
        return self._direct_route_api_key

    @property
    def deployment_id(self) -> Optional[str]:
        # accounts/<account_id>/deployments/<deployment_id>
        return self._id

    @property
    def deployment_name(self) -> Optional[str]:
        """
        Properly converts provided deployment name to a full resource name if
        only the ID was provided.

        If this model is a PEFT addon, the deployment name will be the deployed model name.

        NOTE: Does not query the API to get the full resource name if deployment
        name was not provided on instantiation.
        """
        if self._id is None:
            return None
        if self._id.startswith("accounts/"):
            return self._id
        if (
            self.is_peft_addon()
            and self.deployment_strategy == "on-demand-lora"
            or self.deployment_strategy == "serverless-lora"
        ):
            return f"accounts/{self._gateway.account_id()}/deployedModels/{self._id}"
        return f"accounts/{self._gateway.account_id()}/deployments/{self._id}"

    @property
    def base_deployment_id(self) -> Optional[str]:
        # accounts/<account_id>/deployments/<base_deployment_id>
        return self._base_id

    @property
    def base_deployment_name(self):
        """
        Used for on-demand-lora deployments.

        The base deployment name to use. If not provided, the base deployment
        name will be queried by looking for base models with the same display
        name. Display names are generated from file names or provided explicitly
        through deployment_display_name. This is useful when you want to specify
        a base deployment for LoRA addons.
        """
        if self._base_id is None:
            return None
        if self._base_id.startswith("accounts/"):
            return self._base_id
        return f"accounts/{self._gateway.account_id()}/deployments/{self._base_id}"

    @property
    def temperature(self):
        return self._temperature

    @property
    def perf_metrics_in_response(self):
        return self._perf_metrics_in_response

    @property
    def model(self):
        if not self._model.startswith("accounts/fireworks/models/") and "/" not in self._model:
            model = f"accounts/fireworks/models/{self._model}"
            if self.is_model_on_fireworks_account(model):
                return model
        if not self._model.startswith("accounts/"):
            account_id = self._gateway.account_id()
            return f"accounts/{account_id}/models/{self._model}"
        return self._model

    @staticmethod
    def _validate_model_for_gpu(model: str, accelerator_type: AcceleratorTypeEnum):
        """
        Models are not always supported on all GPU types. This function checks if the model is supported on a GPU.
        """
        if "qwen3-1p7b" in model:
            supported_accelerators = [
                AcceleratorTypeEnum.NVIDIA_H100_80GB,
                AcceleratorTypeEnum.NVIDIA_H200_141GB,
            ]
            if accelerator_type not in supported_accelerators:
                raise ValueError(
                    f"Qwen3-1p7b is not supported on {accelerator_type}. "
                    f"Please pass one of the following accelerators: {list(map(str, supported_accelerators))} "
                    f"to the LLM constructor using the accelerator_type parameter.\n\n"
                    f"Example:\n"
                    f"    from fireworks import LLM\n"
                    f"    llm = LLM(\n"
                    f'        model="qwen3-1p7b",\n'
                    f'        accelerator_type="NVIDIA_H100_80GB"\n'
                    f"    )"
                )

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self._gateway.__aexit__(exc_type, exc_val, exc_tb)

    @property
    def deployment_display_name(self) -> str:
        """
        IMPORTANT: This is used to identify a deployment in the SDK.

        This is the name that will be used to display the deployment in the UI.

        Not sure if we should really be using deployment name for querying
        existing deployments but this way we don't have to worry about name
        collisions since display name is not a unique identifier for
        deployments. Name collisions are important to avoid because you might
        want to have multiple deployments with different parameters in the same
        file/notebook/namespace. So display name is really treated like a
        namespace.
        """
        return self._get_deployment_display_name()

    @sync_cache
    def _get_deployment_display_name(self):
        """
        If a name was specified, deployment name will be the specified name.
        Otherwise, the deployment name will be generated from the filename of the caller where this LLM was instantiated.

        In Jupyter notebooks, we'll use the actual notebook filename rather than the temporary execution file.

        For serverless models, deployment display name is not required since no deployment is created,
        so we return a default name if caller context cannot be determined.
        """
        if self._deployment_display_name is not None:
            return self._deployment_display_name

        # Check if running in a Jupyter notebook environment
        try:
            # Check for Jupyter environment via environment variable
            notebook_path = os.environ.get("JPY_SESSION_NAME")

            if notebook_path:
                logger.debug(f"Found notebook path from environment: {notebook_path}")
                notebook_filename = os.path.basename(notebook_path)
                logger.debug(f"Extracted notebook filename: {notebook_filename}")
                return notebook_filename
        except Exception as e:
            logger.debug(f"Error getting notebook name from environment: {str(e)}")
            pass

        # Get fireworks package path and Python stdlib path
        import fireworks

        package_path = os.path.dirname(fireworks.__file__)
        stdlib_path = sysconfig.get_path("stdlib")

        stack = inspect.stack()
        for frame_info in stack:
            filename = frame_info.filename
            # Skip frames from our package, Python stdlib, other libraries, and internal Python
            if (
                not filename.startswith(package_path)
                and not filename.startswith(stdlib_path)
                and not "/site-packages/" in filename
                and not filename.startswith("<")
            ):
                logger.debug(f"Found caller outside of library code to generate name: {filename}")
                return os.path.basename(filename)

        # If no caller found, we assume this is a REPL session and return a default name
        return "repl-session"

    def apply(self, wait: bool = True):
        """
        Like Terraform apply, this will ensure the deployment is ready and return the deployment.
        """
        self._ensure_deployment_ready(wait=wait)
        return self

    def _with_modified_params(self, **kwargs) -> "LLM":
        """
        Helper method to create a new LLM instance with modified parameters.

        Args:
            **kwargs: Parameters to override in the new LLM instance

        Returns:
            A new LLM instance with the specified parameters modified
        """
        # Try to get api_key and base_url from the client
        api_key = self._client.api_key
        base_url = self._client.base_url
        # Get long_prompt_optimized if it exists
        long_prompt_optimized = getattr(self._auto_tune, "long_prompt_optimized", None)

        # Default parameters from current instance
        params = {
            "model": self._model,
            "deployment_type": self.deployment_type,
            "id": self._id,
            "deployment_display_name": self._deployment_display_name,
            "api_key": api_key,
            "base_url": str(base_url),
            "max_retries": self.max_retries,
            "accelerator_type": self._accelerator_type_original,
            "scale_up_window": self._autoscaling_policy.scale_up_window,
            "scale_down_window": self._autoscaling_policy.scale_down_window,
            "scale_to_zero_window": self._autoscaling_policy.scale_to_zero_window,
            "enable_metrics": self.enable_metrics,
            "region": self._region,
            "description": self._description,
            "annotations": self._annotations,
            "min_replica_count": self._min_replica_count,
            "max_replica_count": self._max_replica_count,
            "replica_count": self._replica_count,
            "accelerator_count": self._accelerator_count,
            "precision": self._precision,
            "world_size": self._world_size,
            "generator_count": self._generator_count,
            "disaggregated_prefill_count": self._disaggregated_prefill_count,
            "disaggregated_prefill_world_size": self._disaggregated_prefill_world_size,
            "max_batch_size": self._max_batch_size,
            "cluster": self._cluster,
            "enable_addons": self._enable_addons,
            "draft_token_count": self._draft_token_count,
            "draft_model": self._draft_model,
            "ngram_speculation_length": self._ngram_speculation_length,
            "max_peft_batch_size": self._max_peft_batch_size,
            "kv_cache_memory_pct": self._kv_cache_memory_pct,
            "enable_session_affinity": self._enable_session_affinity,
            "direct_route_api_keys": self._direct_route_api_keys,
            "num_peft_device_cached": self._num_peft_device_cached,
            "direct_route_type": self._direct_route_type,
            "direct_route_handle": self._direct_route_handle,
            "long_prompt_optimized": long_prompt_optimized,
            "temperature": self._temperature,
            "perf_metrics_in_response": self._perf_metrics_in_response,
        }

        # Override with provided parameters
        params.update(kwargs)

        return LLM(**params)

    def with_deployment_type(self, deployment_type: DeploymentTypeLiteral) -> "LLM":
        """
        Returns a new LLM with the deployment type set to the given value.
        """
        return self._with_modified_params(deployment_type=deployment_type)

    def with_temperature(self, temperature: Optional[float]) -> "LLM":
        """
        Returns a new LLM with the temperature set to the given value.
        """
        return self._with_modified_params(temperature=temperature)

    def with_perf_metrics_in_response(self, perf_metrics_in_response: Optional[bool]) -> "LLM":
        """
        Returns a new LLM with the perf_metrics_in_response set to the given value.
        """
        return self._with_modified_params(perf_metrics_in_response=perf_metrics_in_response)

    def get_time_to_last_token_mean(self) -> Optional[float]:
        """
        Returns the mean time to last token for non-streaming requests. If no metrics are available, returns None.
        """
        if not self.enable_metrics:
            raise ValueError("Metrics are not enabled for this LLM")
        return self._metrics.get_metric_mean("time_to_last_token")

    def is_available_on_serverless(self):
        """
        Checks if the model is available on serverless.
        """
        return self.is_model_available_on_serverless(self.model)

    def delete_deployment(self, ignore_checks: bool = False, wait: bool = True):
        """
        If there is an existing deployment, delete it.
        """
        deployment = self.get_deployment()
        if deployment is None:
            logger.debug("No deployment found to delete")
            return
        if self.is_peft_addon():
            if not isinstance(deployment, SyncDeployedModel):
                raise ValueError(f"Deployment {deployment.name} is not a deployed model")
            if deployment.state == SyncDeployedModel.State.UNDEPLOYING:
                logger.debug(f"LoRA model {self.model} is already unloading, skipping unloading request")
            else:
                if deployment.state == SyncDeployedModel.State.DEPLOYING:
                    logger.debug(
                        f"LoRA model {self.model} is already deploying, waiting for it to be deployed to delete it"
                    )
                    # we need to wait until the deployment is ready to delete it by waiting for it to be deployed
                    self.apply()
                logger.debug(f"Unloading LoRA model {self.model}, deployment={deployment.name}")
                self._gateway.delete_deployed_model_sync(deployment.name)
            if not wait:
                logger.debug(
                    f"wait=False, not polling for LoRA model unloading completion of deployment {deployment.name}"
                )
                return

            logger.debug(f"Unloading LoRA model {self.model}...")
            start_time = time.time()
            last_log_time = 0

            while True:
                current_time = time.time()

                # Try to get the deployed model to check if it still exists
                deployed_model = self._gateway.get_deployed_model_sync(deployment.name)

                if deployed_model is None:
                    # Deployed model no longer exists, deletion is complete
                    total_time = time.time() - start_time
                    logger.debug(f"Serverless LoRA model {self.model} deleted successfully!")
                    logger.debug(f"Deletion completed in {total_time:.2f} seconds!")
                    logger.debug(f"Deployed model {deployment.name} no longer exists (deleted successfully)")
                    break

                # Log detailed info every 10 seconds
                if current_time - last_log_time >= 10:
                    elapsed_so_far = current_time - start_time
                    state_name = SyncDeployedModel.State.Name(deployed_model.state)
                    logger.debug(
                        f"Waiting for deployed model {deployed_model.name} to be deleted, "
                        f"current state: {state_name}, elapsed time: {elapsed_so_far:.2f}s"
                    )
                    last_log_time = current_time

                # Wait before checking again
                time.sleep(2)

            return
        else:
            if not wait:
                logger.debug(f"wait=False, not polling for deployment deletion completion of {deployment.name}")
                return

            logger.debug(f"Deleting deployment {deployment.name}")
            # hard=True so we can reuse the same deployment ID
            self._gateway.delete_deployment_sync(deployment.name, ignore_checks, hard=True)
            # spin until deployment is deleted
            start_time = time.time()
            while deployment is not None:
                current_time = time.time()
                if current_time - start_time >= 10:
                    logger.debug(f"Waiting for deployment {deployment.name} to be deleted...")
                    start_time = current_time
                deployment = self.get_deployment()
                time.sleep(1)

    def _compute_deployment_strategy(self) -> DeploymentStrategyLiteral:
        """
        Determine whether to use serverless or dedicated deployment based on
        model availability, configured deployment type, and model type.

        Returns:
            DeploymentStrategyLiteral: The deployment strategy to use.
        """
        logger.debug(
            f"Computing deployment strategy for model {self.model} with deployment_type {self.deployment_type}"
        )

        if self.is_available_on_serverless():
            if self.deployment_type == "serverless" or self.deployment_type == "auto":
                logger.debug(f"Model {self.model} is available on serverless, using serverless deployment")
                return "serverless"
            else:
                logger.debug(
                    f"Model {self.model} is available on serverless, but deployment_type is on-demand, continuing to ensure deployment is ready"
                )
                return "on-demand"
        else:
            # Check if model supports serverless deployment
            if self.deployment_type in ["serverless", "auto", "on-demand-lora"] and self.is_peft_addon():
                if self.deployment_type == "on-demand-lora":
                    logger.debug(f"Model {self.model} is a PEFT model, using on-demand-lora deployment")
                    return "on-demand-lora"
                logger.debug(f"Checking if model {self.model} supports serverless LoRA")

                # If model supports serverless LoRA, use serverless deployment
                if self.supports_serverless_lora():
                    logger.debug(f"Model {self.model} supports serverless LoRA, using serverless deployment")
                    return "serverless-lora"

                # If deployment type is explicitly serverless but model doesn't support it, raise error
                if self.deployment_type == "serverless":
                    raise ValueError(
                        f"Model {self.model} is not Serverless LoRA enabled and deployment_type is serverless. "
                        f'Use deployment_type of "auto" or "on-demand" instead to deploy this LoRA onto an on-demand deployment.'
                    )

                if self.deployment_type == "auto":
                    """
                    If the model is not available on serverless, but the deployment type is auto,
                    we will deploy the base model with addons enabled, and load the LoRA onto it.
                    This is useful for models that are not available on serverless, but only want to use a single deployment
                    to save on GPU costs. If you really want dedicated resources, then use deployment_type="on-demand".
                    """
                    return "on-demand-lora"

            # Default to on-demand deployment if model is not available on serverless
            logger.debug(
                f"Model {self.model} is not available on serverless, continuing to check for existing deployment"
            )
            return "on-demand"

    def is_model_on_fireworks_account(self, model: str) -> Optional[SyncModel]:
        """
        Check if the model is on the fireworks account.
        If it is, return the model object.
        If it is not, return None.
        """
        models = self.list_fireworks_models()
        model_obj = next((m for m in models if m.name == model), None)
        return model_obj

    @sync_cache
    def is_model_available_on_serverless(self, model: str) -> bool:
        logger.debug(f"Checking if {model} is available on serverless")
        model_obj = self.is_model_on_fireworks_account(model)
        if model_obj is None:
            return False
        logger.debug(f"Found model {model} on under fireworks account")
        is_serverless = self.is_model_deployed_on_serverless_account(model_obj)
        logger.debug(f"Model {model} is {'serverless' if is_serverless else 'not serverless'}")
        return is_serverless

    @sync_cache
    def list_fireworks_models(self) -> List[SyncModel]:
        """
        Find all models on the fireworks account. Does not change often, so we
        embed the Gateway response as a variable in another file and return that
        instead. See the comment on what query is used to get the models.
        """
        result = []
        page_token = None
        while True:
            response = self._gateway.list_models_sync(
                parent=f"accounts/fireworks",
                page_size=200,
                page_token=page_token if page_token else "",
                include_deployed_model_refs=True,
            )
            result.extend(response.models)

            if not response.next_page_token:
                break
            page_token = response.next_page_token

        return result

    def is_model_deployed_on_serverless_account(self, model: SyncModel) -> bool:
        """
        Check if the model is deployed on a serverless-enabled account.

        Args:
            model: The model object to check

        Returns:
            bool: True if the model is deployed on a supported serverless account, False otherwise
        """
        if model.deployed_model_refs:
            for ref in model.deployed_model_refs:
                if (
                    hasattr(ref, "state")
                    and (ref.state == SyncDeployedModel.State.DEPLOYED or ref.state == "DEPLOYED")
                    and hasattr(ref, "deployment")
                    and ref.deployment
                ):
                    # Check if deployment is on a supported account
                    if (
                        ref.deployment.startswith("accounts/fireworks/")
                        or ref.deployment.startswith("accounts/yi-01-ai/")
                        or ref.deployment.startswith("accounts/sentientfoundation/")
                    ):
                        return True
        return False

    @log_execution_time
    def get_deployment(self) -> Optional[Union[SyncDeployment, SyncDeployedModel]]:
        """
        1. If a deployment ID is provided, return the deployment with that name.
        2. If deployment with that ID does not exist, return None.
        3. If model is a PEFT model and is not using live merge, return and deployed model with the same model
        4. Otherwise, look for deployment by model and display name
        """
        if self.is_peft_addon() and (
            self.deployment_strategy == "on-demand-lora" or self.deployment_strategy == "serverless-lora"
        ):
            if self.deployment_name is not None:
                deployed_model = self._gateway.get_deployed_model_sync(self.deployment_name)
                if deployed_model is None:
                    return None
                if deployed_model.model != self.model:
                    raise ValueError(f"Deployed model {self.deployment_name} is not for model {self.model}")
                return deployed_model
            request = SyncListDeployedModelsRequest(filter=f'model="{self.model}"')
            deployed_models = self._gateway.list_deployed_models_sync(request)
            deployed_model = next(iter(deployed_models.deployed_models), None)
            if deployed_model is None:
                return None
            return deployed_model

        if self.deployment_name is not None:
            deployment = self._gateway.get_deployment_sync(self.deployment_name)
            if deployment is None:
                return None
            if deployment.base_model != self.model:
                raise ValueError(f"Deployment {self.deployment_name} is not for model {self.model}")
            return deployment
        deployments = self._gateway.list_deployments_sync(
            filter=f'display_name="{self.deployment_display_name}" AND base_model="{self.model}"'
        )
        deployment = next(iter(deployments), None)
        return deployment

    def supports_serverless_lora(self) -> bool:
        loras = self._gateway.list_serverless_lora_sync()
        if self.peft_base_model is None:
            return False
        base_model_supported = any(lora_model == self.peft_base_model for lora_model in loras.models)
        if not base_model_supported:
            return False
        is_base_model_serverless = self.is_model_available_on_serverless(self.peft_base_model)
        if not is_base_model_serverless:
            return False
        return True

    @log_execution_time
    def _ensure_deployment_ready(self, wait: bool = True) -> None:
        """
        Ensure this LLM is deployed in any way, otherwise raise an error if it
        can't be deployed based on the deployment_type.
        """
        if self.deployment_strategy == "serverless":
            # no deployment needed, just return
            return
        deployment = self.get_deployment()
        if self.deployment_strategy == "serverless-lora":
            if deployment is not None:
                deployed_model = deployment
            else:
                request = SyncCreateDeployedModelRequest()
                request.deployed_model.display_name = self.deployment_display_name
                request.deployed_model.model = self.model
                deployed_model = self._gateway.create_deployed_model_sync(request)

            if deployed_model.state == SyncDeployedModel.State.DEPLOYED:
                return

            logger.debug(f"Deploying serverless LoRA model {self.model}...")

            if not wait:
                logger.debug(f"wait=False, not polling for deployment completion of {deployed_model.name}")
                return

            start_time = time.time()
            last_log_time = 0

            while deployed_model.state != SyncDeployedModel.State.DEPLOYED:
                current_time = time.time()

                # Log detailed info every 10 seconds
                if current_time - last_log_time >= 10:
                    elapsed_so_far = current_time - start_time
                    state_name = SyncDeployedModel.State.Name(deployed_model.state)
                    logger.debug(
                        f"Waiting for deployed model {deployed_model.name} to be ready, "
                        f"current state: {state_name}, elapsed time: {elapsed_so_far:.2f}s"
                    )
                    last_log_time = current_time

                # Check if deployment failed
                if deployed_model.state == SyncDeployedModel.State.STATE_UNSPECIFIED:
                    raise ValueError(f"Deployed model {deployed_model.name} failed to deploy")

                # Wait before checking again
                time.sleep(2)

                # Get updated deployed model state
                updated_deployed_model = self._gateway.get_deployed_model_sync(deployed_model.name)
                if updated_deployed_model is None:
                    raise ValueError(
                        f"Deployed model {deployed_model.name} was unexpectedly deleted during deployment"
                    )
                deployed_model = updated_deployed_model

            total_time = time.time() - start_time
            logger.debug(f"✅ Serverless LoRA model {self.model} deployed successfully!")
            logger.debug(f"✅ Deployment completed in {total_time:.2f} seconds!")
        elif self.deployment_strategy == "on-demand-lora":
            # Validate that a base model is specified for LoRA deployment
            if self.peft_base_model is None:
                raise ValueError(f"PEFT base model is not set for {self.model}")

            # Handle existing deployment if one exists
            if deployment is not None:
                logger.debug(f"Found existing deployment {deployment.name}")

                # Ensure the deployment is a deployed model, not a deployment
                if isinstance(deployment, SyncDeployment):
                    logger.debug(f"Deployment {deployment.name} is a deployment, not a deployed model")
                    raise ValueError(f"Deployment {deployment.name} is a deployment, not a deployed model")

                # Verify the deployment is for the correct model
                if deployment.model != self.model:
                    logger.debug(
                        f"Deployment {deployment.name} model {deployment.model} does not match expected model {self.model}"
                    )
                    raise ValueError(f"Existing deployment with name {deployment.name} is not for model {self.model}")

                # Validate base deployment name matches if specified
                if self.base_deployment_name is not None and deployment.deployment != self.base_deployment_name:
                    logger.debug(
                        f"Deployment {deployment.name} base deployment {deployment.deployment} "
                        f"does not match specified base deployment {self.base_deployment_name}"
                    )
                    raise ValueError(
                        f"Existing deployment with base deployment {deployment.name} "
                        f"does not match the provided base deployment name "
                        f"{self.base_deployment_name}. You should probably just use "
                        f"the existing deployment instead of creating a new one by "
                        f"removing base_deployment_name from LLM instantiation."
                    )

                if deployment.display_name != self.deployment_display_name:
                    logger.debug(
                        f'Deployment "{deployment.name}" has display name '
                        f'"{deployment.display_name}", but "{self.deployment_display_name}" '
                        f"was expected. Will reuse existing deployment as there is no "
                        f"functional difference in creating a new deployment"
                    )

                # Skip deployment if already successfully deployed
                if deployment.state == SyncDeployedModel.State.DEPLOYED:
                    logger.debug(f"Deployment {deployment.name} is already deployed, skipping")
                    return

                # Clean up failed deployments
                if deployment.state == SyncDeployedModel.State.UNDEPLOYING:
                    logger.debug(
                        f"Deployment {deployment.name} is in {SyncDeployment.State.Name(deployment.state)} state, deleting it"
                    )

                    # NOTE: does not respect `wait` argument since we need to
                    # actually wait until the deployed model is unloaded to
                    # trigger a create deployment call
                    self.delete_deployment()

                    logger.debug(f"Successfully deleted failed deployment {deployment.name}")
                    deployment = self.get_deployment()
                    assert deployment is None

            # Deploy the base model first with addons enabled
            base_model_llm = LLM(
                deployment_type="on-demand",
                enable_addons=True,
                model=self.peft_base_model,
                id=self.base_deployment_id,
                min_replica_count=self._min_replica_count,
                max_replica_count=self._max_replica_count,
                replica_count=self._replica_count,
                accelerator_count=self._accelerator_count,
                world_size=self._world_size,
                generator_count=self._generator_count,
                region=self._region,
                annotations=self._annotations,
                description=self._description,
            )
            base_model_llm.apply(wait=wait)
            base_model_deployment = base_model_llm.get_deployment()

            # Verify base model deployment was successful
            if base_model_deployment is None:
                raise ValueError(f"Base model {self.peft_base_model} is not deployed")

            if deployment is not None and deployment.state == SyncDeployedModel.State.DEPLOYING:
                logger.debug(f"LoRA model {self.model} is already deploying, skipping deployment")
                deployed_model = deployment
            else:
                # Create and deploy the LoRA model on top of the base model
                request = SyncCreateDeployedModelRequest()
                request.deployed_model.display_name = self.deployment_display_name
                request.deployed_model.model = self.model
                request.deployed_model.deployment = base_model_deployment.name
                deployed_model = self._gateway.create_deployed_model_sync(request)

            # Monitor the LoRA model deployment progress
            logger.debug(f"Loading LoRA model {self.model} onto base model deployment {base_model_deployment.name}")

            if not wait:
                logger.debug(f"wait=False, not polling for LoRA model loading completion of {deployed_model.name}")
                return

            start_time = time.time()
            last_log_time = 0

            while True:
                current_time = time.time()

                # Poll for deployment status
                deployed_model = self._gateway.get_deployed_model_sync(deployed_model.name)

                # Handle unexpected deletion during deployment
                if deployed_model is None:
                    raise ValueError(
                        f"Deployed model {request.deployed_model.name} was unexpectedly deleted during deployment"
                    )

                # Exit loop when deployment is successful
                if deployed_model.state == SyncDeployedModel.State.DEPLOYED:
                    total_time = time.time() - start_time
                    logger.debug(f"LoRA model {self.model} loaded successfully!")
                    logger.debug(f"Loading completed in {total_time:.2f} seconds!")
                    break

                # Handle deployment failure
                if deployed_model.state == SyncDeployedModel.State.STATE_UNSPECIFIED:
                    raise ValueError(f"Deployed model {deployed_model.name} failed to deploy")

                # Log progress every 10 seconds
                if current_time - last_log_time >= 10:
                    elapsed_so_far = current_time - start_time
                    state_name = SyncDeployedModel.State.Name(deployed_model.state)
                    logger.debug(
                        f"Waiting for deployed model {deployed_model.name} to be ready, "
                        f"current state: {state_name}, elapsed time: {elapsed_so_far:.2f}s"
                    )
                    last_log_time = current_time

                # Wait before next status check
                time.sleep(2)
        elif self.deployment_strategy == "on-demand":

            if deployment is not None and deployment.state == SyncDeployment.State.FAILED:
                logger.debug(f"Deployment {deployment.name} is in FAILED state, deleting it")
                self.delete_deployment()
                logger.debug(f"Successfully deleted failed deployment {deployment.name}")
                deployment = None

            if deployment is None or deployment.state == SyncDeployment.State.DELETED:
                logger.debug(f"No existing deployment found, creating deployment for {self.model}")
                deployment_proto = SyncDeployment(
                    display_name=self.deployment_display_name,
                    base_model=self.model,
                    autoscaling_policy=self._autoscaling_policy_sync,
                )
                if self._accelerator_type_sync is not None:
                    deployment_proto.accelerator_type = self._accelerator_type_sync
                if self._accelerator_count is not None:
                    deployment_proto.accelerator_count = self._accelerator_count
                if self._precision is not None:
                    deployment_proto.precision = getattr(SyncDeployment.Precision, self._precision)
                if self._world_size is not None:
                    deployment_proto.world_size = self._world_size
                if self._generator_count is not None:
                    deployment_proto.generator_count = self._generator_count
                if self._region is not None:
                    deployment_proto.placement.region = getattr(SyncRegion, self._region)
                if self._multi_region is not None:
                    deployment_proto.placement.multi_region = getattr(SyncMultiRegion, self._multi_region)
                if self._description is not None:
                    deployment_proto.description = self._description
                if self._annotations is not None:
                    deployment_proto.annotations.update(self._annotations)
                if self._min_replica_count is not None:
                    deployment_proto.min_replica_count = self._min_replica_count
                else:
                    # default to 0 if not specified
                    deployment_proto.min_replica_count = 0
                if self._max_replica_count is not None:
                    deployment_proto.max_replica_count = self._max_replica_count
                if self._replica_count is not None:
                    deployment_proto.replica_count = self._replica_count
                if self._disaggregated_prefill_count is not None:
                    deployment_proto.disaggregated_prefill_count = self._disaggregated_prefill_count
                if self._disaggregated_prefill_world_size is not None:
                    deployment_proto.disaggregated_prefill_world_size = self._disaggregated_prefill_world_size
                if self._max_batch_size is not None:
                    deployment_proto.max_batch_size = self._max_batch_size
                if self._cluster is not None:
                    deployment_proto.cluster = self._cluster
                if self._enable_addons is not None:
                    deployment_proto.enable_addons = self._enable_addons
                if self._draft_token_count is not None:
                    deployment_proto.draft_token_count = self._draft_token_count
                if self._draft_model is not None:
                    deployment_proto.draft_model = self._draft_model
                if self._ngram_speculation_length is not None:
                    deployment_proto.ngram_speculation_length = self._ngram_speculation_length
                if self._max_peft_batch_size is not None:
                    deployment_proto.max_peft_batch_size = self._max_peft_batch_size
                if self._kv_cache_memory_pct is not None:
                    deployment_proto.kv_cache_memory_pct = self._kv_cache_memory_pct
                if self._enable_session_affinity is not None:
                    deployment_proto.enable_session_affinity = self._enable_session_affinity
                if self._direct_route_api_keys is not None:
                    deployment_proto.direct_route_api_keys.extend(self._direct_route_api_keys)
                if self._num_peft_device_cached is not None:
                    deployment_proto.num_peft_device_cached = self._num_peft_device_cached
                if self._direct_route_type is not None:
                    deployment_proto.direct_route_type = getattr(SyncDirectRouteType, self._direct_route_type)
                if self._direct_route_handle is not None:
                    deployment_proto.direct_route_handle = self._direct_route_handle
                if self._auto_tune_sync.long_prompt:
                    deployment_proto.auto_tune.long_prompt = self._auto_tune_sync.long_prompt
                created_deployment = self._gateway.create_deployment_sync(deployment_proto, self.deployment_id)
                logger.debug(f"Deployment {created_deployment.name} created, waiting for it to be ready")

                if not wait:
                    logger.debug(f"wait=False, not polling for deployment readiness of {created_deployment.name}")
                    return

                # poll deployment status until it's ready
                start_time = time.time()
                last_log_time = 0
                while created_deployment.state != SyncDeployment.State.READY:
                    current_time = time.time()
                    # wait for 9 seconds
                    time.sleep(9)
                    created_deployment = self._gateway.get_deployment_sync(created_deployment.name)
                    if created_deployment is None:
                        raise ValueError(f"Deployment is not found anymore, it was likely deleted while being created")
                    if created_deployment.state == DeploymentState.FAILED:
                        raise ValueError(f"Deployment {created_deployment.name} failed to be created")
                    if created_deployment.state == SyncDeployment.State.DELETED:
                        raise ValueError(f"Deployment {created_deployment.name} was deleted while being created")
                    if created_deployment.state == SyncDeployment.State.DELETING:
                        raise ValueError(f"Deployment {created_deployment.name} is being deleted while being created")
                    if current_time - last_log_time >= 10:
                        elapsed_so_far = current_time - start_time
                        logger.debug(
                            f"Waiting for deployment {created_deployment.name} to be ready, current state: {SyncDeployment.State.Name(created_deployment.state)}, elapsed time: {elapsed_so_far:.2f}s"
                        )
                        last_log_time = current_time

                total_time = time.time() - start_time
                logger.debug(
                    f"Deployment {created_deployment.name} state is READY, checking replicas now (Became READY in {total_time:.2f} seconds)"
                )
            elif isinstance(deployment, SyncDeployment):
                logger.debug(f"Deployment {deployment.name} already exists, checking if it needs to be scaled up")

                field_mask = SyncFieldMask()

                # if autoscaling policy is not equal, update it
                if not self._is_autoscaling_policy_equal(deployment):
                    logger.debug(
                        f"Updating autoscaling policy for {deployment.name} to "
                        f"{self._autoscaling_policy.scale_up_window.total_seconds()}s up, "
                        f"{self._autoscaling_policy.scale_down_window.total_seconds()}s down, "
                        f"{self._autoscaling_policy.scale_to_zero_window.total_seconds()}s to zero"
                    )
                    deployment.autoscaling_policy.scale_up_window = self._autoscaling_policy.scale_up_window  # type: ignore
                    deployment.autoscaling_policy.scale_down_window = self._autoscaling_policy.scale_down_window  # type: ignore
                    deployment.autoscaling_policy.scale_to_zero_window = self._autoscaling_policy.scale_to_zero_window  # type: ignore
                    field_mask.paths.append("autoscaling_policy")

                for field, value in [
                    ("region", self._region),
                    ("description", self._description),
                    ("annotations", self._annotations),
                    ("min_replica_count", self._min_replica_count),
                    ("max_replica_count", self._max_replica_count),
                    ("replica_count", self._replica_count),
                    ("accelerator_count", self._accelerator_count),
                    ("precision", self._precision),
                    ("world_size", self._world_size),
                    ("generator_count", self._generator_count),
                    ("disaggregated_prefill_count", self._disaggregated_prefill_count),
                    ("disaggregated_prefill_world_size", self._disaggregated_prefill_world_size),
                    ("max_batch_size", self._max_batch_size),
                    ("cluster", self._cluster),
                    ("enable_addons", self._enable_addons),
                    ("draft_token_count", self._draft_token_count),
                    ("draft_model", self._draft_model),
                    ("ngram_speculation_length", self._ngram_speculation_length),
                    ("max_peft_batch_size", self._max_peft_batch_size),
                    ("kv_cache_memory_pct", self._kv_cache_memory_pct),
                    ("enable_session_affinity", self._enable_session_affinity),
                    ("direct_route_api_keys", self._direct_route_api_keys),
                    ("num_peft_device_cached", self._num_peft_device_cached),
                    ("direct_route_type", self._direct_route_type),
                    ("direct_route_handle", self._direct_route_handle),
                ]:
                    if value is not None and getattr(deployment, field) != value:
                        logger.debug(f"Updating {field} for {deployment.name} to {value}")
                        setattr(deployment, field, value)
                        field_mask.paths.append(field)

                if (
                    deployment.accelerator_type != self._accelerator_type_sync
                    and self._accelerator_type_sync is not None
                ):
                    raise ValueError(
                        f'Deployment "{deployment.name}" has accelerator type {SyncAcceleratorType.Name(deployment.accelerator_type)}, '
                        f"but LLM has accelerator type {self._accelerator_type}. Please use a different deployment_display_name "
                        f"to use a different accelerator type, as accelerator type cannot be changed for existing deployments."
                    )

                if len(field_mask.paths) > 0:
                    logger.debug(f"Updating deployment {deployment.name} with {field_mask}")
                    start_time = time.time()
                    self._gateway.update_deployment_sync(deployment, field_mask)

                    if not wait:
                        logger.debug(f"wait=False, not polling for deployment update completion of {deployment.name}")
                    else:
                        # poll until deployment is ready
                        while deployment.state != DeploymentState.READY:
                            time.sleep(1)
                            deployment = self._gateway.get_deployment_sync(deployment.name)
                            if deployment is None:
                                raise ValueError(
                                    f"Deployment is not found anymore, it was likely deleted while being updated"
                                )

                        elapsed_time = time.time() - start_time
                        logger.debug(f"Deployment update completed in {elapsed_time:.2f} seconds")

                if deployment.replica_count == 0:
                    logger.debug(f"Deployment {deployment.name} is not ready, scaling to 1 replica")
                    start_time = time.time()
                    self._gateway.scale_deployment_sync(deployment.name, 1)
                    # also send an inference request to wake up the deployment
                    _ = self.chat.completions.acreate(
                        messages=[{"role": "user", "content": 'Say "OK".'}], skip_setup=True
                    )

                    if not wait:
                        logger.debug(
                            f"wait=False, not polling for deployment scale up completion of {deployment.name}"
                        )
                        return

                    # Poll until deployment has at least one replica
                    last_log_time = 0
                    while deployment.replica_count == 0:
                        current_time = time.time()
                        time.sleep(1)
                        deployment = self._gateway.get_deployment_sync(deployment.name)
                        if deployment is None:
                            raise ValueError(
                                f"Deployment is not found anymore, it was likely deleted while being scaled up"
                            )
                        if deployment.state == SyncDeployment.State.FAILED:
                            raise ValueError(f"Deployment {deployment.name} failed to scale up")
                        if deployment.state == SyncDeployment.State.DELETED:
                            raise ValueError(f"Deployment {deployment.name} was deleted while scaling up")
                        if deployment.state == SyncDeployment.State.DELETING:
                            raise ValueError(f"Deployment {deployment.name} is being deleted while scaling up")
                        if current_time - last_log_time >= 10:
                            elapsed_so_far = current_time - start_time
                            logger.debug(
                                f"Waiting for deployment {deployment.name} to scale up, current replicas: {deployment.replica_count}, elapsed time: {elapsed_so_far:.2f}s"
                            )
                            last_log_time = current_time
                            try:
                                self._gateway.scale_deployment_sync(deployment.name, 1)
                            except Exception as e:
                                logger.error(f"Failed to scale deployment {deployment.name}: {e}")

                    total_scale_time = time.time() - start_time
                    logger.debug(f"Deployment {deployment.name} scaled up in {total_scale_time:.2f} seconds")
                logger.debug(f"Deployment {deployment.name} is ready, using deployment")
        else:
            raise ValueError(f"Invalid deployment strategy: {self.deployment_strategy}")

    def scale_to_zero(self) -> Optional[SyncDeployment]:
        """
        Sends a request to scale the deployment to 0 replicas but does not wait for it to complete.
        """
        deployment = self.get_deployment()
        if deployment is None:
            return None
        if isinstance(deployment, SyncDeployedModel):
            raise ValueError(
                f"Deployment {deployment.name} is a LoRA add-on, not a deployment. Please use the deployment_name "
                f"argument to specify a deployment name."
            )
        self._gateway.scale_deployment_sync(deployment.name, 0)
        return deployment

    def scale_to_1_replica(self):
        """
        Scales the deployment to at least 1 replica.
        """
        deployment = self.get_deployment()
        if deployment is None:
            return
        self._gateway.scale_deployment_sync(deployment.name, 1)

    def _is_autoscaling_policy_equal(self, deployment: Union[Deployment, SyncDeployment]) -> bool:
        if isinstance(deployment, SyncDeployment):
            return (
                deployment.autoscaling_policy.scale_up_window == self._autoscaling_policy_sync.scale_up_window
                and deployment.autoscaling_policy.scale_down_window == self._autoscaling_policy_sync.scale_down_window
                and deployment.autoscaling_policy.scale_to_zero_window
                == self._autoscaling_policy_sync.scale_to_zero_window
            )
        return (
            deployment.autoscaling_policy.scale_up_window == self._autoscaling_policy.scale_up_window
            and deployment.autoscaling_policy.scale_down_window == self._autoscaling_policy.scale_down_window
            and deployment.autoscaling_policy.scale_to_zero_window == self._autoscaling_policy.scale_to_zero_window
        )

    @sync_cache
    def get_model(self) -> Optional[SyncModel]:
        return self._gateway.get_model_sync(self.model)

    def is_peft_addon(self) -> bool:
        model = self.get_model()
        if model is None:
            return False
        return model.kind == SyncModel.Kind.HF_PEFT_ADDON

    @property
    def peft_base_model(self) -> Optional[str]:
        """
        If this LLM is a PEFT addon, returns the name of the base model.
        """
        if not self.is_peft_addon():
            return None
        model = self.get_model()
        if model is None:
            return None
        return model.peft_details.base_model

    def list_models(self) -> List[SyncModel]:
        result = []
        page_token = None
        while True:
            response = self._gateway.list_models_sync(
                page_size=200, page_token=page_token if page_token else "", include_deployed_model_refs=True
            )
            result.extend(response.models)

            if not response.next_page_token:
                break
            page_token = response.next_page_token

        return result

    @property
    @sync_cache
    def deployment_url(self) -> Optional[str]:
        """
        Returns the URL to the deployment.
        """
        deployment = self.get_deployment()
        if deployment is None:
            return None
        # get end of "/" split
        id = deployment.name.split("/")[-1]
        if isinstance(deployment, SyncDeployment):
            return f"https://app.fireworks.ai/dashboard/deployments/{id}"
        return f"https://app.fireworks.ai/dashboard/deployedModels/{id}"

    @sync_cache
    def model_id(self):
        """
        Returns the model ID, which is the model name plus the deployment name
        if it exists. This is used for the "model" arg when calling the model.
        """
        if self.is_available_on_serverless() and self.deployment_strategy == "serverless":
            return self.model
        deployment = self.get_deployment()
        if isinstance(deployment, SyncDeployedModel):
            return deployment.name  # accounts/<account_id>/deployedModels/<id>
        if deployment is None:
            if self.deployment_strategy == "on-demand":
                raise ValueError(
                    f"Model {self.model} is not available on serverless and no deployment exists. Make sure to call apply() before calling model_id() or call the model to trigger a deployment."
                )
            else:
                return self.model
        return f"{self.model}#{deployment.name}"

    @property
    def addons_enabled(self) -> bool:
        return False if self._enable_addons is None else self._enable_addons

    @property
    def reinforcement_step(self):
        """Perform a reinforcement learning step."""
        from fireworks.llm.llm_reinforcement_step import LLMReinforcementStep

        return LLMReinforcementStep(self).reinforcement_step

    def __str__(self):
        return self.__repr__()

    def __repr__(self):
        attrs = [f"model={self.model}"]
        if self.deployment_url is not None:
            attrs.append(f"deployment_url={self.deployment_url}")
        if self._deployment_display_name is not None:
            attrs.append(f"deployment_display_name={self._deployment_display_name}")
        if self.deployment_type is not None:
            attrs.append(f"deployment_type={self.deployment_type}")
        if self._accelerator_type is not None:
            attrs.append(f"accelerator_type={self._accelerator_type}")
        if self._region is not None:
            attrs.append(f"region={self._region}")
        if self._min_replica_count is not None:
            attrs.append(f"min_replica_count={self._min_replica_count}")
        if self._max_replica_count is not None:
            attrs.append(f"max_replica_count={self._max_replica_count}")
        if self._replica_count is not None:
            attrs.append(f"replica_count={self._replica_count}")
        if self._accelerator_count is not None:
            attrs.append(f"accelerator_count={self._accelerator_count}")
        if self._precision is not None:
            attrs.append(f"precision={self._precision}")
        if self._world_size is not None:
            attrs.append(f"world_size={self._world_size}")
        if self._generator_count is not None:
            attrs.append(f"generator_count={self._generator_count}")
        if self._disaggregated_prefill_count is not None:
            attrs.append(f"disaggregated_prefill_count={self._disaggregated_prefill_count}")
        if self._disaggregated_prefill_world_size is not None:
            attrs.append(f"disaggregated_prefill_world_size={self._disaggregated_prefill_world_size}")
        if self._max_peft_batch_size is not None:
            attrs.append(f"max_peft_batch_size={self._max_peft_batch_size}")
        if self._kv_cache_memory_pct is not None:
            attrs.append(f"kv_cache_memory_pct={self._kv_cache_memory_pct}")
        if self._enable_session_affinity is not None:
            attrs.append(f"enable_session_affinity={self._enable_session_affinity}")
        if self._direct_route_api_keys is not None:
            attrs.append(f"direct_route_api_keys={self._direct_route_api_keys}")
        if self._num_peft_device_cached is not None:
            attrs.append(f"num_peft_device_cached={self._num_peft_device_cached}")
        if self._direct_route_type is not None:
            attrs.append(f"direct_route_type={self._direct_route_type}")
        if self._direct_route_handle is not None:
            attrs.append(f"direct_route_handle={self._direct_route_handle}")
        if self._temperature is not None:
            attrs.append(f"temperature={self._temperature}")
        if self._perf_metrics_in_response is not None:
            attrs.append(f"perf_metrics_in_response={self._perf_metrics_in_response}")
        if self._enable_addons is not None:
            attrs.append(f"enable_addons={self._enable_addons}")
        return f"LLM({', '.join(attrs)})"

    def create_reinforcement_fine_tuning_job(
        self,
        id: str,
        dataset_or_id: Union[Dataset, str],
        reward_function: Callable,
        evaluation_dataset_or_id: Optional[Union[Dataset, str]] = None,
        # BaseTrainingConfig fields
        output_model: Optional[str] = None,
        jinja_template: Optional[str] = None,
        learning_rate: Optional[float] = 0.0001,
        max_context_length: Optional[int] = 8192,
        lora_rank: Optional[int] = 8,
        base_model_weight_precision: Optional[WeightPrecisionLiteral] = None,
        accelerator_type: Optional[AcceleratorTypeLiteral] = None,
        accelerator_count: Optional[int] = None,
        region: Optional[RegionLiteral] = None,
        epochs: Optional[int] = 5,
        batch_size: Optional[int] = 32768,
        is_intermediate: Optional[bool] = None,
        # WandbConfig fields
        wandb_config: Optional[WandbConfigParam] = None,
        # InferenceParameters fields with defaults from command
        max_tokens: Optional[int] = None,
        temperature: Optional[float] = 1.0,
        top_p: Optional[float] = 1.0,
        n: Optional[int] = 8,
        extra_body: Optional[str] = None,
        top_k: Optional[int] = None,
    ):
        if not is_valid_resource_name(id):
            raise ValueError("id must only contain lowercase a-z, 0-9, and hyphen (-)")

        dataset = dataset_or_id if isinstance(dataset_or_id, Dataset) else Dataset.from_id(dataset_or_id)

        evaluation_dataset = (
            evaluation_dataset_or_id
            if isinstance(evaluation_dataset_or_id, Dataset)
            else Dataset.from_id(evaluation_dataset_or_id) if evaluation_dataset_or_id is not None else None
        )
        if evaluation_dataset is not None:
            evaluation_dataset.sync()

        evaluator = Evaluator(
            gateway=self._gateway,
            reward_function=reward_function,
        )
        evaluator.sync()
        training_config = SyncBaseTrainingConfig(
            output_model=(
                output_model
                if output_model is not None
                else generate_model_resource_name(self._gateway.account_id(), id)
            ),
            jinja_template=jinja_template,
            learning_rate=learning_rate,
            max_context_length=max_context_length,
            lora_rank=lora_rank,
            base_model_weight_precision=base_model_weight_precision,
            accelerator_type=accelerator_type,
            accelerator_count=accelerator_count,
            region=region,
            epochs=epochs,
            batch_size=batch_size,
            is_intermediate=is_intermediate if is_intermediate is not None else False,
        )
        if self.is_peft_addon():
            training_config.warm_start_from = self.model
        else:
            training_config.base_model = self.model
        wandb_config = SyncWandbConfig(
            api_key=wandb_config["api_key"],
            project=wandb_config["project"],
            entity=wandb_config["entity"],
        )
        inference_parameters = SyncInferenceParameters(
            max_tokens=max_tokens,
            temperature=temperature,
            top_p=top_p,
            n=n,
            extra_body=extra_body,
            top_k=top_k,
        )
        proto = SyncReinforcementFineTuningJob(
            display_name=id,
            dataset=dataset.name,
            evaluation_dataset=evaluation_dataset.name if evaluation_dataset is not None else None,
            evaluator=evaluator.name,
            training_config=training_config,
            wandb_config=wandb_config,
            inference_parameters=inference_parameters,
        )

        job = ReinforcementFineTuningJob(
            llm=self,
            proto=proto,
            dataset_or_id=dataset_or_id,
            evaluation_dataset_or_id=evaluation_dataset_or_id,
            api_key=self._gateway._api_key,
            id=id,
        )
        job = job.sync()
        logger.debug(f"See the reinforcement fine-tuning job at {job.url}")
        return job

    def create_supervised_fine_tuning_job(
        self,
        display_name: str,
        dataset_or_id: Union[Dataset, str],
        evaluation_dataset_or_id: Optional[Union[Dataset, str]] = None,
        epochs: Optional[int] = None,
        learning_rate: Optional[float] = None,
        lora_rank: Optional[int] = None,
        jinja_template: Optional[str] = None,
        early_stop: Optional[bool] = None,
        max_context_length: Optional[int] = None,
        base_model_weight_precision: Optional[WeightPrecisionLiteral] = None,
        wandb_config: Optional[WandbConfigParam] = None,
        accelerator_type: Optional[AcceleratorTypeLiteral] = None,
        accelerator_count: Optional[int] = None,
        is_turbo: Optional[bool] = None,
        eval_auto_carveout: Optional[bool] = None,
        region: Optional[RegionLiteral] = None,
        nodes: Optional[int] = None,
        batch_size: Optional[int] = None,
        output_model: Optional[str] = None,
    ) -> "SupervisedFineTuningJob":
        """
        Creates a fine-tuning job for this dataset. If the fine-tuning job already exists, it will block until the job is ready.

        Args:
            display_name: The display name of the fine-tuning job. Must only contain lowercase a-z, 0-9, and hyphen (-).
                The display name is also used to identify the fine-tuning job when listing fine-tuning jobs.
            dataset_or_id: The dataset instance to fine-tune on or the dataset id to fine-tune on.
            epochs: The number of epochs to fine-tune for.
            learning_rate: The learning rate to use for fine-tuning.
            lora_rank: The rank to use for LoRA fine-tuning.
            jinja_template: The Jinja template to use for formatting the dataset.
            early_stop: Whether to enable early stopping.
            max_context_length: The maximum context length to use during fine-tuning.
            base_model_weight_precision: The weight precision to use for the base model.
            wandb_config: Configuration for Weights & Biases integration.
            evaluation_dataset: Dataset or dataset id to use for evaluation.
            accelerator_type: The type of accelerator to use for fine-tuning.
            accelerator_count: The number of accelerators to use.
            is_turbo: Whether to use turbo mode for faster fine-tuning.
            eval_auto_carveout: Whether to automatically carve out evaluation data.
            region: The region to run the fine-tuning job in.
            nodes: The number of nodes to use for distributed training.
            batch_size: The batch size to use during training.
            output_model: The name of the output model to create. If not provided, it will be the same as the name argument.

        Returns:
            Fine-tuned LLM
        """
        # Import here to avoid circular import
        from fireworks.supervised_fine_tuning_job import SupervisedFineTuningJob

        if display_name is None:
            raise ValueError("display_name is required")
        if not is_valid_resource_name(display_name):
            raise ValueError("job name must only contain lowercase a-z, 0-9, and hyphen (-)")

        dataset = dataset_or_id if isinstance(dataset_or_id, Dataset) else Dataset.from_id(dataset_or_id)
        evaluation_dataset = (
            evaluation_dataset_or_id
            if isinstance(evaluation_dataset_or_id, Dataset)
            else Dataset.from_id(evaluation_dataset_or_id) if evaluation_dataset_or_id is not None else None
        )

        if wandb_config is not None:
            wandb_config = SyncWandbConfig(
                api_key=wandb_config["api_key"],
                project=wandb_config["project"],
                entity=wandb_config["entity"],
            )
        unique_name = f"{display_name}-{str(uuid4())[:5].lower()}"
        proto = SyncSupervisedFineTuningJob(
            name=f"accounts/{self._gateway.account_id()}/supervisedFineTuningJobs/{unique_name}",
            display_name=display_name,
            epochs=epochs,
            learning_rate=learning_rate,
            lora_rank=lora_rank,
            jinja_template=jinja_template,
            max_context_length=max_context_length,
            base_model_weight_precision=base_model_weight_precision,
            wandb_config=wandb_config,
            dataset=dataset.name,
            evaluation_dataset=evaluation_dataset.name if evaluation_dataset is not None else None,
            accelerator_type=accelerator_type,
            accelerator_count=accelerator_count,
            region=region,
            nodes=nodes,
            batch_size=batch_size,
            output_model=output_model,
        )
        if self.is_peft_addon():
            proto.warm_start_from = self.model
        else:
            proto.base_model = self.model
        if early_stop is not None:
            proto.early_stop = early_stop
        if is_turbo is not None:
            proto.is_turbo = is_turbo
        if eval_auto_carveout is not None:
            proto.eval_auto_carveout = eval_auto_carveout

        job = SupervisedFineTuningJob(
            llm=self,
            proto=proto,
            dataset_or_id=dataset_or_id,
            evaluation_dataset_or_id=evaluation_dataset_or_id,
            api_key=self._gateway._api_key,
        )
        job = job.sync()
        logger.debug(f"See the fine-tuning job at {job.url}")
        return job



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: gateway/reinforcement_fine_tuning_epoch.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'gateway/reinforcement_fine_tuning_epoch.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import batch_inference_job_pb2 as gateway_dot_batch__inference__job__pb2
from . import job_progress_pb2 as gateway_dot_job__progress__pb2
from . import options_pb2 as gateway_dot_options__pb2
from . import status_pb2 as gateway_dot_status__pb2
from . import training_pb2 as gateway_dot_training__pb2
from . import wandb_pb2 as gateway_dot_wandb__pb2
from google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from google.api import resource_pb2 as google_dot_api_dot_resource__pb2
from google.api import visibility_pb2 as google_dot_api_dot_visibility__pb2
from google.protobuf import field_mask_pb2 as google_dot_protobuf_dot_field__mask__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n-gateway/reinforcement_fine_tuning_epoch.proto\x12\x07gateway\x1a!gateway/batch_inference_job.proto\x1a\x1agateway/job_progress.proto\x1a\x15gateway/options.proto\x1a\x14gateway/status.proto\x1a\x16gateway/training.proto\x1a\x13gateway/wandb.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x19google/api/resource.proto\x1a\x1bgoogle/api/visibility.proto\x1a google/protobuf/field_mask.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\x95\x08\n\x1cReinforcementFineTuningEpoch\x12\x13\n\x04name\x18\x01 \x01(\tB\x05\xe2\x41\x02\x03\x05\x12\x36\n\x0b\x63reate_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x05\xe2\x41\x02\x03\x05\x12\x39\n\x0e\x63ompleted_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x05\xe2\x41\x02\x03\x05\x12\x15\n\x07\x64\x61taset\x18\x04 \x01(\tB\x04\xe2\x41\x01\x02\x12 \n\x12\x65valuation_dataset\x18\x05 \x01(\tB\x04\xe2\x41\x01\x01\x12 \n\x12\x65val_auto_carveout\x18\x06 \x01(\x08\x42\x04\xe2\x41\x01\x01\x12\'\n\x05state\x18\x07 \x01(\x0e\x32\x11.gateway.JobStateB\x05\xe2\x41\x02\x03\x05\x12&\n\x06status\x18\x08 \x01(\x0b\x32\x0f.gateway.StatusB\x05\xe2\x41\x02\x03\x05\x12\x19\n\ncreated_by\x18\t \x01(\tB\x05\xe2\x41\x02\x03\x05\x12\x35\n\x0bupdate_time\x18\n \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x04\xe2\x41\x01\x03\x12:\n\x0ftraining_config\x18\x0b \x01(\x0b\x32\x1b.gateway.BaseTrainingConfigB\x04\xe2\x41\x01\x01\x12\x17\n\tevaluator\x18\x0c \x01(\tB\x04\xe2\x41\x01\x02\x12\x30\n\x0cwandb_config\x18\r \x01(\x0b\x32\x14.gateway.WandbConfigB\x04\xe2\x41\x01\x01\x12\x17\n\trollout_n\x18\x0e \x01(\x05\x42\x04\xe2\x41\x01\x01\x12\'\n\x19\x65nding_assistant_messages\x18\x0f \x01(\tB\x04\xe2\x41\x01\x01\x12\x44\n reinforcement_fine_tuning_job_id\x18\x10 \x01(\tB\x1a\xe2\x41\x01\x01\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12G\n\x0cjob_progress\x18\x11 \x01(\x0b\x32\x14.gateway.JobProgressB\x1b\xe2\x41\x02\x03\x05\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12@\n\x14inference_parameters\x18\x12 \x01(\x0b\x32\x1c.gateway.InferenceParametersB\x04\xe2\x41\x01\x01:\xd4\x01\xea\x41\xc1\x01\n-api.fireworks.ai/ReinforcementFineTuningEpoch\x12Saccounts/{AccountId}/reinforcementFineTuningEpochs/{ReinforcementFineTuningEpochId}*\x1dreinforcementFineTuningEpochs2\x1creinforcementFineTuningEpoch\x82\xf1\x04\x0b\n\x07\x41\x63\x63ount\x18\x01\"q\n&GetReinforcementFineTuningEpochRequest\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12\x33\n\tread_mask\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.FieldMaskB\x04\xe2\x41\x01\x01\"\xaa\x02\n)CreateReinforcementFineTuningEpochRequest\x12\x14\n\x06parent\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12T\n\x1freinforcement_fine_tuning_epoch\x18\x02 \x01(\x0b\x32%.gateway.ReinforcementFineTuningEpochB\x04\xe2\x41\x01\x02\x12)\n\x05\x64\x65\x62ug\x18\x03 \x01(\x08\x42\x1a\xe2\x41\x01\x01\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12\x30\n\"reinforcement_fine_tuning_epoch_id\x18\x04 \x01(\tB\x04\xe2\x41\x01\x01\x12\x34\n\x10skip_validations\x18\x05 \x01(\x08\x42\x1a\xe2\x41\x01\x01\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\"\xd6\x01\n(ListReinforcementFineTuningEpochsRequest\x12\x14\n\x06parent\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12\x17\n\tpage_size\x18\x02 \x01(\x05\x42\x04\xe2\x41\x01\x01\x12\x18\n\npage_token\x18\x03 \x01(\tB\x04\xe2\x41\x01\x01\x12\x14\n\x06\x66ilter\x18\x04 \x01(\tB\x04\xe2\x41\x01\x01\x12\x16\n\x08order_by\x18\x05 \x01(\tB\x04\xe2\x41\x01\x01\x12\x33\n\tread_mask\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.FieldMaskB\x04\xe2\x41\x01\x01\"\xa9\x01\n)ListReinforcementFineTuningEpochsResponse\x12O\n reinforcement_fine_tuning_epochs\x18\x01 \x03(\x0b\x32%.gateway.ReinforcementFineTuningEpoch\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t\x12\x12\n\ntotal_size\x18\x03 \x01(\x05\"?\n)DeleteReinforcementFineTuningEpochRequest\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x42\x43ZAgithub.com/fw-ai/fireworks/control_plane/protos/generated/gatewayb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'gateway.reinforcement_fine_tuning_epoch_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'ZAgithub.com/fw-ai/fireworks/control_plane/protos/generated/gateway'
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['name']._loaded_options = None
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['name']._serialized_options = b'\342A\002\003\005'
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['create_time']._loaded_options = None
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['create_time']._serialized_options = b'\342A\002\003\005'
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['completed_time']._loaded_options = None
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['completed_time']._serialized_options = b'\342A\002\003\005'
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['dataset']._loaded_options = None
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['dataset']._serialized_options = b'\342A\001\002'
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['evaluation_dataset']._loaded_options = None
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['evaluation_dataset']._serialized_options = b'\342A\001\001'
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['eval_auto_carveout']._loaded_options = None
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['eval_auto_carveout']._serialized_options = b'\342A\001\001'
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['state']._loaded_options = None
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['state']._serialized_options = b'\342A\002\003\005'
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['status']._loaded_options = None
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['status']._serialized_options = b'\342A\002\003\005'
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['created_by']._loaded_options = None
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['created_by']._serialized_options = b'\342A\002\003\005'
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['update_time']._loaded_options = None
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['update_time']._serialized_options = b'\342A\001\003'
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['training_config']._loaded_options = None
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['training_config']._serialized_options = b'\342A\001\001'
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['evaluator']._loaded_options = None
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['evaluator']._serialized_options = b'\342A\001\002'
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['wandb_config']._loaded_options = None
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['wandb_config']._serialized_options = b'\342A\001\001'
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['rollout_n']._loaded_options = None
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['rollout_n']._serialized_options = b'\342A\001\001'
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['ending_assistant_messages']._loaded_options = None
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['ending_assistant_messages']._serialized_options = b'\342A\001\001'
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['reinforcement_fine_tuning_job_id']._loaded_options = None
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['reinforcement_fine_tuning_job_id']._serialized_options = b'\342A\001\001\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['job_progress']._loaded_options = None
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['job_progress']._serialized_options = b'\342A\002\003\005\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['inference_parameters']._loaded_options = None
  _globals['_REINFORCEMENTFINETUNINGEPOCH'].fields_by_name['inference_parameters']._serialized_options = b'\342A\001\001'
  _globals['_REINFORCEMENTFINETUNINGEPOCH']._loaded_options = None
  _globals['_REINFORCEMENTFINETUNINGEPOCH']._serialized_options = b'\352A\301\001\n-api.fireworks.ai/ReinforcementFineTuningEpoch\022Saccounts/{AccountId}/reinforcementFineTuningEpochs/{ReinforcementFineTuningEpochId}*\035reinforcementFineTuningEpochs2\034reinforcementFineTuningEpoch\202\361\004\013\n\007Account\030\001'
  _globals['_GETREINFORCEMENTFINETUNINGEPOCHREQUEST'].fields_by_name['name']._loaded_options = None
  _globals['_GETREINFORCEMENTFINETUNINGEPOCHREQUEST'].fields_by_name['name']._serialized_options = b'\342A\001\002'
  _globals['_GETREINFORCEMENTFINETUNINGEPOCHREQUEST'].fields_by_name['read_mask']._loaded_options = None
  _globals['_GETREINFORCEMENTFINETUNINGEPOCHREQUEST'].fields_by_name['read_mask']._serialized_options = b'\342A\001\001'
  _globals['_CREATEREINFORCEMENTFINETUNINGEPOCHREQUEST'].fields_by_name['parent']._loaded_options = None
  _globals['_CREATEREINFORCEMENTFINETUNINGEPOCHREQUEST'].fields_by_name['parent']._serialized_options = b'\342A\001\002'
  _globals['_CREATEREINFORCEMENTFINETUNINGEPOCHREQUEST'].fields_by_name['reinforcement_fine_tuning_epoch']._loaded_options = None
  _globals['_CREATEREINFORCEMENTFINETUNINGEPOCHREQUEST'].fields_by_name['reinforcement_fine_tuning_epoch']._serialized_options = b'\342A\001\002'
  _globals['_CREATEREINFORCEMENTFINETUNINGEPOCHREQUEST'].fields_by_name['debug']._loaded_options = None
  _globals['_CREATEREINFORCEMENTFINETUNINGEPOCHREQUEST'].fields_by_name['debug']._serialized_options = b'\342A\001\001\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_CREATEREINFORCEMENTFINETUNINGEPOCHREQUEST'].fields_by_name['reinforcement_fine_tuning_epoch_id']._loaded_options = None
  _globals['_CREATEREINFORCEMENTFINETUNINGEPOCHREQUEST'].fields_by_name['reinforcement_fine_tuning_epoch_id']._serialized_options = b'\342A\001\001'
  _globals['_CREATEREINFORCEMENTFINETUNINGEPOCHREQUEST'].fields_by_name['skip_validations']._loaded_options = None
  _globals['_CREATEREINFORCEMENTFINETUNINGEPOCHREQUEST'].fields_by_name['skip_validations']._serialized_options = b'\342A\001\001\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_LISTREINFORCEMENTFINETUNINGEPOCHSREQUEST'].fields_by_name['parent']._loaded_options = None
  _globals['_LISTREINFORCEMENTFINETUNINGEPOCHSREQUEST'].fields_by_name['parent']._serialized_options = b'\342A\001\002'
  _globals['_LISTREINFORCEMENTFINETUNINGEPOCHSREQUEST'].fields_by_name['page_size']._loaded_options = None
  _globals['_LISTREINFORCEMENTFINETUNINGEPOCHSREQUEST'].fields_by_name['page_size']._serialized_options = b'\342A\001\001'
  _globals['_LISTREINFORCEMENTFINETUNINGEPOCHSREQUEST'].fields_by_name['page_token']._loaded_options = None
  _globals['_LISTREINFORCEMENTFINETUNINGEPOCHSREQUEST'].fields_by_name['page_token']._serialized_options = b'\342A\001\001'
  _globals['_LISTREINFORCEMENTFINETUNINGEPOCHSREQUEST'].fields_by_name['filter']._loaded_options = None
  _globals['_LISTREINFORCEMENTFINETUNINGEPOCHSREQUEST'].fields_by_name['filter']._serialized_options = b'\342A\001\001'
  _globals['_LISTREINFORCEMENTFINETUNINGEPOCHSREQUEST'].fields_by_name['order_by']._loaded_options = None
  _globals['_LISTREINFORCEMENTFINETUNINGEPOCHSREQUEST'].fields_by_name['order_by']._serialized_options = b'\342A\001\001'
  _globals['_LISTREINFORCEMENTFINETUNINGEPOCHSREQUEST'].fields_by_name['read_mask']._loaded_options = None
  _globals['_LISTREINFORCEMENTFINETUNINGEPOCHSREQUEST'].fields_by_name['read_mask']._serialized_options = b'\342A\001\001'
  _globals['_DELETEREINFORCEMENTFINETUNINGEPOCHREQUEST'].fields_by_name['name']._loaded_options = None
  _globals['_DELETEREINFORCEMENTFINETUNINGEPOCHREQUEST'].fields_by_name['name']._serialized_options = b'\342A\001\002'
  _globals['_REINFORCEMENTFINETUNINGEPOCH']._serialized_start=368
  _globals['_REINFORCEMENTFINETUNINGEPOCH']._serialized_end=1413
  _globals['_GETREINFORCEMENTFINETUNINGEPOCHREQUEST']._serialized_start=1415
  _globals['_GETREINFORCEMENTFINETUNINGEPOCHREQUEST']._serialized_end=1528
  _globals['_CREATEREINFORCEMENTFINETUNINGEPOCHREQUEST']._serialized_start=1531
  _globals['_CREATEREINFORCEMENTFINETUNINGEPOCHREQUEST']._serialized_end=1829
  _globals['_LISTREINFORCEMENTFINETUNINGEPOCHSREQUEST']._serialized_start=1832
  _globals['_LISTREINFORCEMENTFINETUNINGEPOCHSREQUEST']._serialized_end=2046
  _globals['_LISTREINFORCEMENTFINETUNINGEPOCHSRESPONSE']._serialized_start=2049
  _globals['_LISTREINFORCEMENTFINETUNINGEPOCHSRESPONSE']._serialized_end=2218
  _globals['_DELETEREINFORCEMENTFINETUNINGEPOCHREQUEST']._serialized_start=2220
  _globals['_DELETEREINFORCEMENTFINETUNINGEPOCHREQUEST']._serialized_end=2283
# @@protoc_insertion_point(module_scope)
